using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de HavingClauseTest.
	/// </summary>
	[TestFixture]
	public class HavingClauseTest
	{
        [TestFixtureSetUp]
        public void Setup()
        {
            QueryFrameworkConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryBuilder.QueryProcessor.ResetProcessor();
        }

		public HavingClauseTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}


		[Test]
		public void EmptyClause()
		{
			HavingClause having = new HavingClause();
			Assert.AreEqual("", having.ToString());

		}

		[Test]
		public void SimpleHaving()
		{
			HavingClause having = new HavingClause();
			having.Filters.Add("Articles.ArticleId = 29");
			Assert.AreEqual("HAVING (Articles.ArticleId = 29)", having.ToString());

			having = new HavingClause();
			having.Filters.Add("Articles.ArticleId", SqlOperator.Equals, 30);
			Assert.AreEqual("HAVING (Articles.ArticleId = 30)", having.ToString());

            having = new HavingClause();
            SearchCondition sc = new SearchCondition();
            sc.Filters.Add("Articles.ArticleId", SqlOperator.Equals, 30);
            having.SearchCondition = sc;
            Assert.AreEqual("HAVING (Articles.ArticleId = 30)", having.ToString());
		}

		[Test]
		public void ComplexHaving()
		{
			HavingClause having = new HavingClause();
			having.Filters.Add("ArticleGroupSetType = 'O'");
			having.Filters.Add("ArticleGroupSets.Active = 1");
            Assert.AreEqual(FilterUnion.And, having.Union);
			having.Union = FilterUnion.And;
            Assert.AreEqual(FilterUnion.And, having.Union);
            having.Union = FilterUnion.Or;
            Assert.AreEqual(FilterUnion.Or, having.Union);
			Assert.AreEqual("HAVING (ArticleGroupSetType = 'O') OR (ArticleGroupSets.Active = 1)", having.ToString());

		}

        [Test]
        public void NegativeHaving()
        {
            HavingClause having = new HavingClause();
            having.Negative = true;
            having.Filters.Add("Articles.ArticleId = 29");
            Assert.AreEqual(true, having.Negative);
            Assert.AreEqual("HAVING NOT (Articles.ArticleId = 29)", having.ToString());
        }
		
	}
}
