using System;
using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Probar la clusula de orden
	/// </summary>
	[TestFixture]
	public class OrderClauseTest
	{
        [TestFixtureSetUp]
        public void Setup()
        {
            QueryFrameworkConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryBuilder.QueryProcessor.ResetProcessor();
        }

		[Test]
		public void TestOrderItem()
		{
			SqlField sqlField = new SqlField("Articles.Name AS ArticleName");
			OrderItem orderItem = new OrderItem(sqlField);
			Assert.AreEqual("Articles.Name ASC", orderItem.ToString());

            orderItem = new OrderItem(sqlField, "ASC");
            Assert.AreEqual(OrderDirection.Asc, orderItem.Direction);
            Assert.AreEqual("Articles.Name ASC", orderItem.ToString());

            orderItem = new OrderItem(sqlField, "asc ");
            Assert.AreEqual(OrderDirection.Asc, orderItem.Direction);
            Assert.AreEqual("Articles.Name ASC", orderItem.ToString());

			orderItem = new OrderItem(sqlField, "DESC");
			Assert.AreEqual(OrderDirection.Desc, orderItem.Direction);
			Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

            orderItem = new OrderItem("Articles.Name", "DESC");
            Assert.AreEqual(OrderDirection.Desc, orderItem.Direction);
            Assert.AreEqual("Articles.Name", orderItem.Field.ToString());
            Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

            orderItem = new OrderItem("Articles.Name", OrderDirection.Desc);
            Assert.AreEqual(OrderDirection.Desc, orderItem.Direction);
            Assert.AreEqual("Articles.Name", orderItem.Field.ToString());
            Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

			orderItem = new OrderItem(sqlField, OrderDirection.Desc);
			Assert.AreEqual(OrderDirection.Desc, orderItem.Direction);
			Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

			orderItem = new OrderItem("SUM(Movements) > 23 DESC");
			Assert.AreEqual(OrderDirection.Desc, orderItem.Direction);
			Assert.AreEqual("SUM(Movements) > 23", orderItem.Field.ToString());

            
		}

        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestInvalidOrderItem()
        {
            OrderItem orderItem = new OrderItem("test", "invalid");
            orderItem.ToString();
        }

        [Test]        
        public void TestEmptyOrderItem()
        {
            OrderItem orderItem = new OrderItem("");
            Assert.AreEqual("", orderItem.ToString());            
        }

		[Test]
		public void EmptyOrderBy()
		{
			OrderByClause orderBy = new OrderByClause();
			Assert.AreEqual("", orderBy.ToString());

            orderBy = new OrderByClause();
            orderBy.Fields.Add("");
            Assert.AreEqual("", orderBy.ToString());
		}

		[Test]
		public void SimpleOrderBy()
		{
			OrderByClause orderBy = new OrderByClause("Articles.Name DESC");
			Assert.AreEqual("ORDER BY Articles.Name DESC", orderBy.ToString());

			orderBy = new OrderByClause();
			orderBy.Fields.Add("Articles.Name DESC");
			Assert.AreEqual("ORDER BY Articles.Name DESC", orderBy.ToString());
		}

		[Test]
		public void MultipleOrderBy()
		{
			OrderByClause orderBy = new OrderByClause("Articles.Name DESC, Articles.Date ASC");
			Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Date ASC", orderBy.ToString());
			
			orderBy = new OrderByClause();
			orderBy.Fields.Add("Articles.Name DESC");
			orderBy.Fields.Add("Articles.Date ASC");
			Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Date ASC", orderBy.ToString());
		}

        [Test]
        public void InsertOrderBy()
        {
            OrderByClause orderBy = new OrderByClause("Articles.Date ASC");
            Assert.AreEqual("ORDER BY Articles.Date ASC", orderBy.ToString());

            orderBy.Fields.Insert(0, "Articles.Name DESC");
            Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Date ASC", orderBy.ToString());

            orderBy.Fields.Insert(1, "Articles.Id ASC");
            Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Id ASC,Articles.Date ASC", orderBy.ToString());
        }
	}
}
