using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de SearchConditionTest.
	/// </summary>
	[TestFixture] 
	public class SearchConditionTest
	{
        [TestFixtureSetUp]
        public void Setup()
        {
            QueryFrameworkConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryBuilder.QueryProcessor.ResetProcessor();
        }

		public SearchConditionTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}

        /// <summary>
        /// Comprueba que una SearchCondition vaca no devuelva nada
        /// </summary>
        [Test]
        public void EmptySearchCondition()
        {
            SearchCondition sc = new SearchCondition();
            sc.ExternalUnion = FilterUnion.And;

            string expectedResult = "";
            Assert.AreEqual(expectedResult, sc.ToString());

            SearchCondition sc2 = new SearchCondition();
            sc.ExternalUnion = FilterUnion.Or;
            sc.Filters.Add(sc2);

            Assert.AreEqual(expectedResult, sc.ToString());            

        }

		[Test]
		public void SimpleSearchCondition()
		{			
			SqlField customerId = new SqlField("CustomerId", null, "Customers");
			SqlOperator sqlOperator = SqlOperator.Equals;
			SqlField paymentId = new SqlField("Payment.CustomerId");				

			
			SearchFilter se = new SearchFilter(customerId, sqlOperator, paymentId);			
			Assert.AreEqual("Customers.CustomerId = Payment.CustomerId", se.ToString());
			
			SearchCondition sc = new SearchCondition();
			sc.Filters.Add(se);			
			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId)", sc.ToString());

			SearchFilter se2 = new SearchFilter("[Products].VendorId = Shop.VendorId");
            se2.ExternalUnion = FilterUnion.And;					
			Assert.AreEqual("[Products].VendorId = Shop.VendorId", se2.ToString());

			sc.Filters.Add(se2);		
			
			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)", sc.ToString());

			SearchFilter se3 = new SearchFilter(new SqlField("CountryId", null, "Shops"), sqlOperator, new SqlField("CountryId", null, "Countries"));
            se3.ExternalUnion = FilterUnion.Or;			

			Assert.AreEqual("Shops.CountryId = Countries.CountryId", se3.ToString());

			sc.Filters.Add(se3);

			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId) OR (Shops.CountryId = Countries.CountryId)", sc.ToString());
            Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId) OR (Shops.CountryId = Countries.CountryId)", sc.GetPredicateExpression());					
		}

		[Test]
		public void SimpleNegativeSearchCondition()
		{
			SearchCondition sc = new SearchCondition();
			sc.Negative = true;
			sc.Filters.Add("CustomerType = 'A'");		
			Assert.AreEqual("NOT (CustomerType = 'A')", sc.ToString());

		}

		[Test]
		public void ComplexNegativeSearchCondition()
		{
			SearchCondition sc = new SearchCondition();
			sc.Filters.Add("CustomerId = 2");
			sc.Filters.Add("Amount", SqlOperator.Greater, 0).Negative = true;
			
			Assert.AreEqual("(CustomerId = 2) AND NOT (Amount > 0)", sc.ToString());

			sc.DefaultFiltersUnion = FilterUnion.Or;

			Assert.AreEqual("(CustomerId = 2) OR NOT (Amount > 0)", sc.ToString());			
		}

		[Test]
		public void ComplexSearchCondition()
		{			
			SqlField customerId = new SqlField("CustomerId", null, "Customers");
			SqlOperator sqlOperator = SqlOperator.Equals;
			SqlField paymentId = new SqlField("Payment.CustomerId");		

			SearchFilter se1 = new SearchFilter(customerId, sqlOperator, paymentId);			
			se1.Negative = false;			

			SearchFilter se2 = new SearchFilter("[Products].VendorId = Shop.VendorId");
            se2.ExternalUnion = FilterUnion.And;			

			SearchFilter se3 = new SearchFilter(new SqlField("CountryId", null, "Shops"), sqlOperator, new SqlField("CountryId", null, "Countries"));
            se3.ExternalUnion = FilterUnion.Or;			

			SearchCondition sc = new SearchCondition();
			SearchCondition sc1 = new SearchCondition();
			SearchCondition sc2 = new SearchCondition();

			sc1.Filters.Add(se1);
			sc1.Filters.Add(se2);
			sc2.Filters.Add(se3);

            sc2.ExternalUnion = FilterUnion.And;

			sc.Filters.Add(sc1);
			sc.Filters.Add(sc2);

			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)", sc1.ToString());
			Assert.AreEqual("(Shops.CountryId = Countries.CountryId)", sc2.ToString());
			Assert.AreEqual("((Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)) AND ((Shops.CountryId = Countries.CountryId))", sc.ToString());
			
		}

		[Test]
		public void SearchConditionSimpleOperators()
		{
			SearchFilter sf = new SearchFilter("ArticleId", SqlOperator.Equals, 1);
			Assert.AreEqual("ArticleId = 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Distinct, 1);
			Assert.AreEqual("ArticleId != 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Greater, 1);
			Assert.AreEqual("ArticleId > 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.GreaterOrEqual, 1);
			Assert.AreEqual("ArticleId >= 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Less, 1);
			Assert.AreEqual("ArticleId < 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.LessOrEqual, 1);
			Assert.AreEqual("ArticleId <= 1", sf.ToString());				


		}

		[Test]
		public void SearchConditionStringOperators()
		{
			SearchFilter sf = new SearchFilter("ArticleId", SqlOperator.Like, "Test%");
			Assert.AreEqual("ArticleId LIKE 'Test%'", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.NotLike, "Test%");
			Assert.AreEqual("ArticleId NOT LIKE 'Test%'", sf.ToString());
		}

		[Test]
		public void SearchConditionComplexOperators()
		{
			SearchFilter sf = new SearchFilter(SqlOperator.Contains, "QuantityPerUnit, 'bottles'");
			Assert.AreEqual("CONTAINS (QuantityPerUnit, 'bottles')", sf.ToString());

			sf = new SearchFilter(SqlOperator.Exists, "SELECT NULL");
			Assert.AreEqual("EXISTS (SELECT NULL)", sf.ToString());

			sf = new SearchFilter("state", SqlOperator.In, "'CA','IN','MD'");
			Assert.AreEqual("state IN ('CA','IN','MD')", sf.ToString());
            
			sf = new SearchFilter("state", SqlOperator.NotIn, "'CA','IN','MD'");
			Assert.AreEqual("state NOT IN ('CA','IN','MD')", sf.ToString());

            sf = new SearchFilter("Year", SqlOperator.In, "(2001, 2002, 2003)");
            Assert.AreEqual("Year IN (2001, 2002, 2003)", sf.ToString());

		}

        [Test]
        public void SearchConditionUnionPreference()
        {
            SearchCondition searchCondition = new SearchCondition();
            searchCondition.Filters.Add("Commited = 1");
            searchCondition.Filters.Add("Active = 0");           
            string sqlCommited = "(Commited = 1) AND (Active = 0)";
            Assert.AreEqual(sqlCommited, searchCondition.ToString());

            searchCondition = new SearchCondition();
            searchCondition.Filters.Add("Commited = 1");
            searchCondition.Filters.Add("Active = 0");
            searchCondition.DefaultFiltersUnion = FilterUnion.NotSet;
            sqlCommited = "(Commited = 1) AND (Active = 0)";
            Assert.AreEqual(sqlCommited, searchCondition.ToString());

            searchCondition = new SearchCondition();
            searchCondition.Filters.Add("Commited = 1");
            searchCondition.Filters.Add("Active = 0");
            searchCondition.DefaultFiltersUnion = FilterUnion.Or;
            sqlCommited = "(Commited = 1) OR (Active = 0)";
            Assert.AreEqual(sqlCommited, searchCondition.ToString());
            

            // Si se indica una unin explcita, no debe sobreescribirla la Union genrica
            searchCondition = new SearchCondition();   
            searchCondition.Filters.Add("Commited = 1");
            searchCondition.Filters.Add(FilterUnion.Or, "Active = 0");
            searchCondition.DefaultFiltersUnion = FilterUnion.And;
            sqlCommited = "(Commited = 1) OR (Active = 0)";
            Assert.AreEqual(sqlCommited, searchCondition.ToString());            

            searchCondition = new SearchCondition();
            searchCondition.Filters.Add("Commited = 1");
            searchCondition.Filters.Add(FilterUnion.NotSet, "Active = 0");
            searchCondition.DefaultFiltersUnion = FilterUnion.Or;
            sqlCommited = "(Commited = 1) OR (Active = 0)";
            Assert.AreEqual(sqlCommited, searchCondition.ToString());


            //additionalSearchCondition.Union = FilterUnion.OR;

            //SearchCondition mainSearchCondition = new SearchCondition();
            //mainSearchCondition.Filters.Add("User = 'Richard'");
            //mainSearchCondition.Filters.Add(additionalSearchCondition);


            //string expectedSql = "(User = 'Richard') AND ((Commited = 1) OR (Active = 0))";
            //string resultSql = mainSearchCondition.ToString();

            //Assert.AreEqual(expectedSql, resultSql);


        }

        [Test]
        public void SearchConditionComposed()
        {
            SearchCondition additionalSearchCondition = new SearchCondition();
            additionalSearchCondition.Filters.Add("Commited = 1");
            additionalSearchCondition.Filters.Add("Active = 0");                        
            string sqlCommited = "(Commited = 1) AND (Active = 0)";
            Assert.AreEqual(sqlCommited, additionalSearchCondition.ToString());            

            additionalSearchCondition.DefaultFiltersUnion = FilterUnion.Or;
            sqlCommited = "(Commited = 1) OR (Active = 0)";
            Assert.AreEqual(sqlCommited, additionalSearchCondition.ToString());

            SearchCondition mainSearchCondition = new SearchCondition();
            mainSearchCondition.Filters.Add("User = 'u'");
            mainSearchCondition.Filters.Add(additionalSearchCondition);            
            sqlCommited = "(User = 'u') AND ((Commited = 1) OR (Active = 0))";
            Assert.AreEqual(sqlCommited, mainSearchCondition.ToString());            

            mainSearchCondition.DefaultFiltersUnion = FilterUnion.Or;
            sqlCommited = "(User = 'u') OR ((Commited = 1) OR (Active = 0))";
            Assert.AreEqual(sqlCommited, mainSearchCondition.ToString());

            mainSearchCondition.Filters.Add(FilterUnion.And, "Visible=False");
            sqlCommited = "(User = 'u') OR ((Commited = 1) OR (Active = 0)) AND (Visible=False)";
            Assert.AreEqual(sqlCommited, mainSearchCondition.ToString());


         }

        [Test]
        public void StringCompareCondition()
        {
            SearchCondition sc = new SearchCondition();            
            sc.Filters.Add("languageCode", SqlOperator.Equals, "iv", ExpressionType.Primitive);
            Assert.AreEqual("(languageCode = 'iv')", sc.ToString());
            sc.Filters.Add("fieldCode", SqlOperator.Equals, "field", ExpressionType.Complex);
            Assert.AreEqual("(languageCode = 'iv') AND (fieldCode = field)", sc.ToString());
        }

	}
}
