using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de SelectClauseTest.
	/// </summary>
	[TestFixture]
	public abstract class SelectClauseTest
	{

	    #region Common Test

        [Test]
		public void SimpleSelect()
		{
			SelectClause select = new SelectClause();
			select.Fields.Add("CustomerId");			
			Assert.AreEqual("SELECT CustomerId", select.ToString());
			select.Fields.Add("CustomerName");
			Assert.AreEqual("SELECT CustomerId,CustomerName", select.ToString());			
		}

		[Test]
		public void SimpleSelectWithStrangeAlias()
		{
			SelectClause select = new SelectClause();
			select.Fields.Add("CustomerId", "Id del cliente");		
			Assert.AreEqual("SELECT CustomerId AS 'Id del cliente'", select.ToString());
		}
        

		[Test]
		public void SelectDistinct()
		{
			SelectClause select = new SelectClause();
			select.Fields.Add("CustomerId");
            select.SelectionType = SelectionTypes.Distinct;
			Assert.AreEqual("SELECT DISTINCT CustomerId", select.ToString());
		}		
        

		[Test]
		public void SelectWithExpression()
		{
			SelectClause select = new SelectClause();	
			select.Fields.Add("SUM(Quantity) AS Total");
			Assert.AreEqual("SELECT SUM(Quantity) AS Total", select.ToString());
		}

		[Test]
		public void SelectWithSubQuery()
		{
			SelectClause select = new SelectClause();	
			select.Fields.Add("(SELECT SUM(Quantity) AS Quantity)", "Total");
			Assert.AreEqual("SELECT (SELECT SUM(Quantity) AS Quantity) AS Total", select.ToString());

			select = new SelectClause();	
			select.Fields.Add("SELECT SUM(Quantity) AS Quantity", "Total");
			Assert.AreEqual("SELECT (SELECT SUM(Quantity) AS Quantity) AS Total", select.ToString());
		}

		[Test]
		public void SelectConstructor()
		{
			SelectClause select = new SelectClause("Articles.ArticleId, Articles.Name as 'Article Name'");
			Assert.AreEqual(2, select.Fields.Count);
			Assert.AreEqual("SELECT Articles.ArticleId,Articles.Name AS 'Article Name'", select.ToString());
			Assert.AreEqual("Articles", select.Fields[0].TableName);
			Assert.AreEqual("ArticleId", select.Fields[0].ColumnNameOrExpression);
			Assert.AreEqual("Articles", select.Fields[1].TableName);
			Assert.AreEqual("Name", select.Fields[1].ColumnNameOrExpression);
			Assert.AreEqual("'Article Name'", select.Fields[1].Alias);

        }

        [Test]
        public void SelectBuilderWithEmptyFields()
        {
            SelectClause select = new SelectClause();
            string sql = "SELECT *";
            Assert.AreEqual(sql, select.ToString());
        }

        #endregion

        #region Particular Tests

        public abstract void SelectTop();

        public abstract void SelectTopDistinct();

        public abstract void SelectEnableCalculateFoundRows();

        #endregion

    }
}
