using System;
using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.Nunit.Common
{
    [TestFixture]
    public class StringExpressionTest
    {
        [TestFixtureSetUp]
        public void Setup()
        {
            QueryFrameworkConfiguration.Dialect = Dialects.MSSql2005Dialect;
            QueryBuilder.QueryProcessor.ResetProcessor();
        }

        [Test]
        public void SimpleStringExpression()
        {
            StringExpression se1 = new StringExpression("Active = 1");
            Assert.AreEqual("Active = 1", se1.ToString());

            se1.Expression = "NotActive = 2";
            Assert.AreEqual("NotActive = 2", se1.ToString());            
        }

        [Test]
        public void TestClone()
        {
            StringExpression se1 = new StringExpression("Active = 1");
            StringExpression se2 = (StringExpression)se1.Clone();
            Assert.AreEqual(se1.Expression, se2.Expression);                        
        }

        [Test]
        public void TestGetStringValue()
        {
            // Booleano            
            Assert.AreEqual("1", QueryExpression.GetStringValue(true));
            Assert.AreEqual("0", QueryExpression.GetStringValue(false));
            // DateTime
            Assert.AreEqual("'19770121 22:23:45'", QueryExpression.GetStringValue(new DateTime(1977, 01, 21, 22, 23, 45)));
            // Null
            Assert.AreEqual("NULL", QueryExpression.GetStringValue(null));
            // String
            Assert.AreEqual("Hola", QueryExpression.GetStringValue("Hola"));
            Assert.AreEqual("'Hola'", QueryExpression.GetStringValue("'Hola'"));
            Assert.AreEqual("'Ho''la'", QueryExpression.GetStringValue("Ho'la", true));

            // Double
            double testDouble = 23.45;
            Assert.AreEqual("23.45", QueryExpression.GetStringValue(testDouble));
            // Float
            float testFloat = 56.78F;
            Assert.AreEqual("56.78", QueryExpression.GetStringValue(testFloat));


        }

    }
}
