using System.Collections.Generic;
using Avanzis.QueryFramework;
using NUnit.Framework;

namespace Avanzis.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de WhereClauseTest.
	/// </summary>
	[TestFixture]
	public abstract class WhereClauseTest
	{

		public WhereClauseTest()
		{
			
		}

		[Test]
		public void EmptyWhere()
		{
			WhereClause where = new WhereClause();
			Assert.AreEqual("", where.ToString());
		}

        [Test]
        public void EmptyWhereWithEmptyFilters()
        {
            WhereClause where = new WhereClause();
            where.Filters.Add("");
            Assert.AreEqual("", where.ToString());

            SearchCondition sc = new SearchCondition();            
            where.Filters.Add(sc);
            Assert.AreEqual("", where.ToString());
        }

        [Test]
        public void NegativeWhere()
        {
            WhereClause where = new WhereClause();
            where.Negative = true;
            where.Filters.Add("Articles.ArticleId = 12");
            Assert.AreEqual(true, where.Negative);
            Assert.AreEqual("WHERE NOT (Articles.ArticleId = 12)", where.ToString());
        }

		[Test]
		public void SimpleWhere()
		{
			WhereClause where = new WhereClause();
			where.Filters.Add("Articles.ArticleId = 29");
			Assert.AreEqual("WHERE (Articles.ArticleId = 29)", where.ToString());

			where = new WhereClause();
			where.Filters.Add("Articles.ArticleId", SqlOperator.Equals, 30);
			Assert.AreEqual("WHERE (Articles.ArticleId = 30)", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Language", SqlOperator.Equals, "es-ES");
            Assert.AreEqual("WHERE (Language = 'es-ES')", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Language", SqlOperator.Equals, "'es-ES'");
            Assert.AreEqual("WHERE (Language = 'es-ES')", where.ToString());

            where = new WhereClause();
            SearchCondition sc = new SearchCondition();
            sc.Filters.Add("Articles.ArticleId", SqlOperator.Equals, 30);
            where.SearchCondition = sc;
            Assert.AreEqual("WHERE (Articles.ArticleId = 30)", where.ToString());
		}

        [Test]
        public void WhereWithNullValues()
        {
            WhereClause where = new WhereClause();            
            where.Filters.Add("FirstName", SqlOperator.Is, null);
            Assert.AreEqual("WHERE (FirstName IS NULL)", where.ToString());

            where = new WhereClause();
            where.Filters.Add("FirstName", SqlOperator.Equals, null);
            Assert.AreEqual("WHERE (FirstName IS NULL)", where.ToString());
        }


        [Test]
        public void WhereWithUnaryOperator()
        {
            WhereClause where = new WhereClause();
            where.Filters.Add("FirstName", SqlOperator.IsNull);
            Assert.AreEqual("WHERE (FirstName IS NULL)", where.ToString());
        }

        [Test]
        public void AdvancedOperatorsInWhereClause()
        {
            WhereClause where = new WhereClause();
            where.Filters.Add("Year", SqlOperator.In, "(2003,2004,2005)");
            Assert.AreEqual("WHERE (Year IN (2003,2004,2005))", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Year", SqlOperator.In, "2003,2004,2005");
            Assert.AreEqual("WHERE (Year IN (2003,2004,2005))", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Subclass", SqlOperator.In, "('N','R','T')");
            Assert.AreEqual("WHERE (Subclass IN ('N','R','T'))", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Subclass", SqlOperator.In, "'N','R','T'");
            Assert.AreEqual("WHERE (Subclass IN ('N','R','T'))", where.ToString());

            where = new WhereClause();
            where.Filters.Add("Subclass", SqlOperator.In, "('N','R','T')");
            Assert.AreEqual("WHERE (Subclass IN ('N','R','T'))", where.ToString());
        }
       

        [Test]
        public void FunctionsInWhereClause()
        {
            WhereClause where = new WhereClause();
            where.Filters.Add("EventDate", SqlOperator.Greater, "GETDATE()");
            Assert.AreEqual("WHERE (EventDate > GETDATE())", where.ToString());

            where = new WhereClause();
            where.Filters.Add("2005", SqlOperator.Greater, "YEAR(EventDate)");
            Assert.AreEqual("WHERE (2005 > YEAR(EventDate))", where.ToString());
        }
        

		[Test]
		public void ComplexWhere()
		{
			WhereClause where = new WhereClause();
			where.Filters.Add("ArticleGroupSetType = 'O'");
			where.Filters.Add("ArticleGroupSets.Active = 1");
            Assert.AreEqual(FilterUnion.And, where.Union);
			where.Union = FilterUnion.And;
            Assert.AreEqual(FilterUnion.And, where.Union);
            where.Union = FilterUnion.Or;
            Assert.AreEqual(FilterUnion.Or, where.Union);
			Assert.AreEqual("WHERE (ArticleGroupSetType = 'O') OR (ArticleGroupSets.Active = 1)", where.ToString());
		}


        [Test]
        public void InWhere()
        {
            WhereClause where = new WhereClause();
            where.Filters.Add("Filter",SqlOperator.In,new List<string>{"a","b"});
            Assert.AreEqual("WHERE (Filter IN ('a','b'))", where.ToString());

            WhereClause where2 = new WhereClause();
            where2.Filters.Add("Filter", SqlOperator.In, new List<int> { 1, 2 });
            Assert.AreEqual("WHERE (Filter IN (1,2))", where2.ToString());
        }

        public abstract void ParametersInWhereClause();
        



        


        
		
		
	}
}
