using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clase bsica para las expresiones
    /// </summary>
    public abstract class QueryExpression : ICloneable
    {
        /// <summary>
        /// Representacin como string del valor NULL
        /// </summary>
        public const string NullString = "NULL";

        /// <summary>
        /// Gets the expression.
        /// </summary>
        /// <returns></returns>
        public abstract string GetExpression();

        /// <summary>
        /// Devuelve un objeto <see cref="T:System.String"/> que representa al objeto <see cref="T:System.Object"/> actual.
        /// </summary>
        /// <returns>
        /// Objeto <see cref="T:System.String"/> que representa el objeto <see cref="T:System.Object"/> actual.
        /// </returns>
        public override string ToString()
        {
            return GetExpression();
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        /// A new object that is a copy of this instance.
        /// </returns>
        public abstract object Clone();

        #endregion

        #region Mtodos Staticos

        /// <summary>
        /// Gets the string value of a Expression
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <returns></returns>
        public static string GetStringValue(object expression)
        {
            return QueryProcessor.Current.GetExpressionStringValue(expression);
        }

        /// <summary>
        /// Gets the string value for a Expression
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <param name="escapeString">if set to <c>true</c> [escape string].</param>
        /// <returns></returns>
        public static string GetStringValue(object expression, bool escapeString)
        {
            return QueryProcessor.Current.GetExpressionStringValue(expression, escapeString);
        }

        /// <summary>
        /// Gets the string value of an expression
        /// </summary>
        /// <param name="expression">The expression.</param>
        /// <param name="expressionType">Type of the expression.</param>
        /// <returns></returns>
        public static string GetStringValue(object expression, ExpressionType expressionType)
        {
            return QueryProcessor.Current.GetExpressionStringValue(expression, expressionType);
        }

        #endregion
    }

    /// <summary>
    /// Tipos de expresiones posibles
    /// </summary>
    public enum ExpressionType
    {
        /// <summary>
        /// The ExpressionType is Unknow
        /// </summary>
        Unknown,
        /// <summary>
        /// The ExpressionType is a primitive value
        /// </summary>
        Primitive,
        /// <summary>
        /// The ExpressionType is a complex value, like the name of a field
        /// </summary>
        Complex
    }
}