using System.Data;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    ///   Parmetro para una consulta
    /// </summary>
    public class QueryParameter : IDataParameter
    {
        #region Variables

        private string _parameterName;

        #endregion

        #region Constructor

        /// <summary>
        /// </summary>
        protected QueryParameter()
        {
            this.SourceVersion = DataRowVersion.Current;
            this.Direction = ParameterDirection.Input;
        }

        ///// <summary>
        ///// 
        ///// </summary>
        ///// <param name="parameterName"></param>
        //public QueryParameter(string parameterName) : this()
        //{
        //    this.ParameterName = parameterName;
        //}

        /// <summary>
        /// </summary>
        /// <param name = "parameterName"></param>
        /// <param name = "parameterValue"></param>
        public QueryParameter(string parameterName, object parameterValue) : this()
        {
            this.ParameterName = parameterName;
            this.Value = parameterValue;
        }

        #endregion

        #region Properties

        /// <summary>
        ///   Gets or sets the name of the parameter
        /// </summary>
        /// <value>The name.</value>
        public virtual string ParameterName
        {
            get { return _parameterName; }
            set { _parameterName = GetParameterName(value); }
        }

        public object Value { get; set; }

        public ParameterDirection Direction { get; set; }

        public DataRowVersion SourceVersion { get; set; }

        public bool IsNullable { get; set; }

        public string SourceColumn { get; set; }

        public DbType DbType { get; set; }

        #endregion

        #region Methods

        /// <summary>
        ///   Returns a <see cref = "T:System.String"></see> that represents the current <see cref = "T:System.Object"></see>.
        /// </summary>
        /// <returns>
        ///   A <see cref = "T:System.String"></see> that represents the current <see cref = "T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return this.ParameterName;
        }

        #endregion

        #region Static Methods

        /// <summary>
        ///   Gets the name of the parameter.
        /// </summary>
        /// <param name = "parameterName">Name of the parameter.</param>
        /// <returns></returns>
        public static string GetParameterName(string parameterName)
        {
            return QueryProcessor.Current.GetParameterName(parameterName);
        }

        #endregion
    }
}