using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Campo de una consulta
    /// </summary>
    public class SqlField : ICloneable
    {
        private string fieldAlias;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="columnNameOrExpression">Nombre de la columna de la base de datos o expresin</param>
        /// <param name="fieldAlias">Alias de este campo</param>
        /// <param name="tableName">Nombre de la tabla a la que pertenece el campo</param>
        public SqlField(string columnNameOrExpression, string fieldAlias, string tableName)
        {
            this.TableName = tableName;
            this.ColumnNameOrExpression = columnNameOrExpression;
            this.Alias = fieldAlias;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="columnNameOrExpression">Nombre de la columna de la base de datos o expresin</param>
        /// <param name="fieldAlias">Alias de este campo</param>		
        public SqlField(string columnNameOrExpression, string fieldAlias)
        {
            QueryProcessor.Current.Parse(this, columnNameOrExpression, fieldAlias);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="columnNameOrExpression">Nombre de la columna de la base de datos o expresin</param>
        public SqlField(string columnNameOrExpression)
        {
            QueryProcessor.Current.Parse(this, columnNameOrExpression);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public SqlField()
        {
        }

        /// <summary>
        /// Nombre de la tabla a la que pertenece el campo
        /// </summary>
        public string TableName { get; set; }

        /// <summary>
        /// Nombre de la columna de la base de datos o expresin
        /// </summary>
        public string ColumnNameOrExpression { get; set; }

        /// <summary>
        /// Alias de este campo
        /// </summary>
        public string Alias
        {
            get { return fieldAlias; }
            set
            {
                if (value != null)
                    fieldAlias = value.Trim();
                else
                    fieldAlias = null;
            }
        }

        /// <summary>
        /// Representacin SQL de este campo
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon del campo actual.
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            SqlField clonedField = new SqlField();
            clonedField.Alias = this.Alias;
            clonedField.ColumnNameOrExpression = this.ColumnNameOrExpression;
            clonedField.TableName = this.TableName;
            return clonedField;
        }

        #endregion
    }
}