using System;
using System.Collections.Specialized;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clase para construir y procesar un ConnectionString
    /// </summary>
    public class ConnectionStringBuilder
    {
        private readonly NameValueCollection connectionStringCollection = new NameValueCollection();

        #region Properties

        /// <summary>
        /// Servidor de la cadena de conexin
        /// </summary>
        public string Server
        {
            get { return connectionStringCollection["server"]; }
            set { connectionStringCollection["server"] = value; }
        }

        /// <summary>
        /// Base de datos de la cadena de conexin
        /// </summary>
        public string Database
        {
            get { return connectionStringCollection["database"]; }
            set { connectionStringCollection["database"] = value; }
        }

        /// <summary>
        /// Usuario
        /// </summary>
        public string User
        {
            get { return connectionStringCollection["user"]; }
            set { connectionStringCollection["user"] = value; }
        }

        /// <summary>
        /// Clave
        /// </summary>
        public string Password
        {
            get { return connectionStringCollection["pwd"]; }
            set { connectionStringCollection["pwd"] = value; }
        }

        /// <summary>
        /// Gets or sets the <see cref="System.String"/> at the specified index.
        /// </summary>
        /// <value></value>
        public string this[string index]
        {
            get { return connectionStringCollection[index]; }
            set { connectionStringCollection[index] = value; }
        }

        /// <summary>
        /// Gets the connection string collection.
        /// </summary>
        /// <value>The connection string collection.</value>
        internal NameValueCollection ConnectionStringCollection
        {
            get { return connectionStringCollection; }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        public ConnectionStringBuilder()
        {
        }

        /// <summary>
        /// Constructor de un ConnectionString
        /// </summary>
        /// <param name="server">Servidor al que se va a conectar</param>
        /// <param name="database">Base de datos</param>
        /// <param name="user">Usuario</param>
        /// <param name="password">Clave del usuario</param>
        public ConnectionStringBuilder(string server, string database, string user, string password)
        {
            connectionStringCollection["server"] = server;
            connectionStringCollection["database"] = database;
            connectionStringCollection["user"] = user;
            connectionStringCollection["pwd"] = password;
        }

        /// <summary>
        /// Constructor de un ConnectionString
        /// </summary>
        /// <param name="connectionString">Cadena de conexin</param>
        public ConnectionStringBuilder(string connectionString)
        {
            if (String.IsNullOrEmpty(connectionString))
                throw new ArgumentNullException("connectionString");

            string[] connectionStringParts = connectionString.Split(new[] {';'});

            foreach (string connectionStringPart in connectionStringParts)
            {
                if (connectionStringPart.Length > 0 && connectionStringPart.IndexOf("=") > 0)
                {
                    string[] connectionParts = connectionStringPart.Split(new[] {'='});
                    string key = connectionParts[0].Trim();
                    if (key.Equals("uid", StringComparison.InvariantCultureIgnoreCase))
                        key = "user";
                    connectionStringCollection[key] = connectionParts[1].Trim();
                }
            }
        }

        /// <summary>
        /// Devuelve la cadena de conexin
        /// </summary>
        /// <returns>string con la cadena de conexin</returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }
    }
}