using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clase para generar consultas DELETE
    /// </summary>
    public class DeleteBuilder : Builder
    {
        private Boolean allowDeleteAll;
        private DeleteClause deleteClause = new DeleteClause();
        private FromClause fromClause = new FromClause();
        private WhereClause whereClause = new WhereClause();

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DeleteBuilder"/> class.
        /// </summary>
        public DeleteBuilder() : this(null, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DeleteBuilder"/> class.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        public DeleteBuilder(string tableName) : this(tableName, false)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DeleteBuilder"/> class.
        /// </summary>
        /// <param name="allowDeleteAll">if set to <c>true</c> [allow delete all].</param>
        public DeleteBuilder(bool allowDeleteAll) : this(null, allowDeleteAll)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:DeleteBuilder"/> class.
        /// </summary>
        /// <param name="tableName">Name of the table.</param>
        /// <param name="allowDeleteAll">if set to <c>true</c> [allow delete all].</param>
        public DeleteBuilder(string tableName, bool allowDeleteAll)
        {
            this.TableName = tableName;
            this.AllowDeleteAll = allowDeleteAll;
        }

        /// <summary>
        /// Gets or sets the name of the table.
        /// </summary>
        /// <value>The name of the table.</value>        
        public override string TableName
        {
            get { return From.TableName; }
            set { From.TableName = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [allow delete all objects from the table].
        /// </summary>
        /// <value><c>true</c> if [allow delete all]; otherwise, <c>false</c>.</value>
        public bool AllowDeleteAll
        {
            get { return allowDeleteAll; }
            set { allowDeleteAll = value; }
        }

        /// <summary>
        /// Gets the delete clause.
        /// </summary>
        /// <value>The delete clause.</value>
        internal DeleteClause DeleteClause
        {
            get { return deleteClause; }
        }

        /// <summary>
        /// Clasula FROM
        /// </summary>
        public FromClause From
        {
            get
            {
                if (fromClause == null)
                    fromClause = new FromClause();
                return fromClause;
            }
            set { fromClause = value; }
        }

        /// <summary>
        /// Clasula WHERE
        /// </summary>
        public WhereClause Where
        {
            get
            {
                if (whereClause == null)
                    whereClause = new WhereClause();
                return whereClause;
            }
            set { whereClause = value; }
        }

        /// <summary>
        /// Gets the filters.
        /// </summary>
        /// <value>The filters.</value>
        public SearchFiltersCollection Filters
        {
            get { return Where.Filters; }
        }


        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }
    }
}