using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Condicin de bsqueda
    /// </summary>
    public class SearchCondition : SearchPredicate, ICloneable
    {
        private SearchFiltersCollection filters;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <remarks>Por defecto las uniones se realizarn del tipo AND</remarks>
        public SearchCondition()
        {
        }

        /// <summary>
        /// Filtros de la condicind e bqueda
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get
            {
                if (filters == null)
                {
                    filters = new SearchFiltersCollection();
                }
                return filters;
            }
        }

        /// <summary>
        /// Comando SQL que representa esta condicin de bsqueda
        /// </summary>
        /// <returns></returns>
        public override string GetPredicateExpression()
        {
            return this.ToString();
        }

        /// <summary>
        /// Comando SQL que representa esta condicin de bsqueda
        /// </summary>
        /// <returns>String con la sentencia SQL</returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la condicin de bsqueda
        /// </summary>
        /// <returns></returns>
        public override object Clone()
        {
            SearchCondition cloned = new SearchCondition();
            cloned.Negative = this.Negative;
            cloned.ExternalUnion = this.ExternalUnion;
            cloned.filters = (SearchFiltersCollection) this.Filters.Clone();
            return cloned;
        }

        #endregion
    }
}