using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Condicin de bsqueda
    /// </summary>
    /// <remarks>Esta es la expresin de bsqueda ms sencilla</remarks>
    public class SearchFilter : SearchPredicate, ICloneable
    {
        internal QueryExpression Expression;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="filter">Expresin de esta condicin</param>
        public SearchFilter(string filter) : this(FilterUnion.NotSet, false, filter)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union">Tipo de unin</param>
        /// <param name="filter">Expresin de esta condicin</param>
        public SearchFilter(FilterUnion union, string filter) : this(union, false, filter)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="negative">Si debe negar la expresin</param>
        /// <param name="filter">Expresin de esta condicin</param>
        public SearchFilter(bool negative, string filter) : this(FilterUnion.NotSet, negative, filter)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union">Tipo de unin</param>
        /// <param name="negative">Si debe negar la expresin</param>
        /// <param name="filter">Expresin de esta condicin</param>
        public SearchFilter(FilterUnion union, bool negative, string filter)
        {
            this.ExternalUnion = union;
            this.Negative = negative;
            Expression = new StringExpression(filter);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SearchFilter"/> class.
        /// </summary>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>        
        public SearchFilter(object leftExpression, SqlOperator compareOperation, object rightExpression) : this(leftExpression, compareOperation, rightExpression, ExpressionType.Unknown)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperation">Operador para comparar las expresiones</param>
        /// <param name="rightExpression"></param>
        /// <param name="rightExpressionType">Tipo de expresin del operador de la derecha</param>
        public SearchFilter(object leftExpression, SqlOperator compareOperation, object rightExpression, ExpressionType rightExpressionType) : this(FilterUnion.NotSet, false, leftExpression, compareOperation, rightExpression, rightExpressionType)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union">Tipo de unin</param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperation"></param>
        /// <param name="rightExpression"></param>
        public SearchFilter(FilterUnion union, object leftExpression, SqlOperator compareOperation, object rightExpression) : this(union, false, leftExpression, compareOperation, rightExpression)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SearchFilter"/> class.
        /// </summary>
        /// <param name="negative">if set to <c>true</c> [negative].</param>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        public SearchFilter(bool negative, object leftExpression, SqlOperator compareOperation, object rightExpression)
            : this(negative, leftExpression, compareOperation, rightExpression, ExpressionType.Complex)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="negative">Si debe negar la expresin</param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperation"></param>
        /// <param name="rightExpression"></param>
        /// <param name="rightExpressionType"></param>
        public SearchFilter(bool negative, object leftExpression, SqlOperator compareOperation, object rightExpression, ExpressionType rightExpressionType) : this(FilterUnion.NotSet, negative, leftExpression, compareOperation, rightExpression, rightExpressionType)
        {
        }


        /// <summary>
        /// Initializes a new instance of the <see cref="T:SearchFilter"/> class.
        /// </summary>
        /// <param name="union">The union.</param>
        /// <param name="negative">if set to <c>true</c> [negative].</param>
        /// <param name="leftExpression">The left expression.</param>
        /// <param name="compareOperation">The compare operation.</param>
        /// <param name="rightExpression">The right expression.</param>
        public SearchFilter(FilterUnion union, bool negative, object leftExpression, SqlOperator compareOperation, object rightExpression)
            : this(union, negative, leftExpression, compareOperation, rightExpression, ExpressionType.Complex)
        {
        }


        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="compareOperation">Operador para comparar las expresiones</param>
        /// <param name="rightExpression"></param>
        public SearchFilter(SqlOperator compareOperation, object rightExpression) : this(FilterUnion.NotSet, false, null, compareOperation, rightExpression)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="negative">Si debe negar la expresin</param>
        /// <param name="compareOperation">Operador para comparar las expresiones</param>
        /// <param name="rightExpression"></param>
        public SearchFilter(bool negative, SqlOperator compareOperation, object rightExpression) : this(FilterUnion.NotSet, negative, compareOperation, rightExpression)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union"></param>
        /// <param name="compareOperation"></param>
        /// <param name="rightExpression"></param>
        public SearchFilter(FilterUnion union, SqlOperator compareOperation, object rightExpression) : this(union, false, compareOperation, rightExpression)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union"></param>
        /// <param name="negative"></param>
        /// <param name="compareOperation"></param>
        /// <param name="rightExpression"></param>
        public SearchFilter(FilterUnion union, bool negative, SqlOperator compareOperation, object rightExpression)
        {
            this.ExternalUnion = union;
            this.Negative = negative;
            this.Expression = new CompareExpression(compareOperation, rightExpression);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="union"></param>
        /// <param name="negative"></param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperation"></param>
        /// <param name="rightExpression"></param>
        /// <param name="rightExpressionType"></param>
        public SearchFilter(FilterUnion union, bool negative, object leftExpression, SqlOperator compareOperation, object rightExpression, ExpressionType rightExpressionType)
        {
            this.ExternalUnion = union;
            this.Negative = negative;
            this.Expression = new CompareExpression(leftExpression, compareOperation, rightExpression, rightExpressionType);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona condicin de bsqueda
        /// </summary>
        /// <returns></returns>
        public override object Clone()
        {
            SearchFilter cloned = new SearchFilter(this.ExternalUnion, this.Negative, this.Expression.ToString());
            return cloned;
        }

        #endregion

        /// <summary>
        /// Obtiene la representacin SQL de esta expresin
        /// </summary>
        /// <returns></returns>
        public override string GetPredicateExpression()
        {
            return Expression.ToString();
        }
    }
}