using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Predicados que forman la condicin de bsqueda
    /// </summary>
    /// <remarks>Clase abstracta de la que heredarn todas
    /// las posibles condiciones de bsqueda</remarks>
    public abstract class SearchPredicate : ICloneable
    {
        /// <summary>
        /// Unin interna de sus filtros
        /// </summary>
        private FilterUnion defaultFiltersUnion = FilterUnion.And;

        private FilterUnion externalUnion = FilterUnion.NotSet;

        /// <summary>
        /// Si debe negar el predicado actual.
        /// 
        /// </summary>
        public bool Negative { get; set; }

        /// <summary>
        /// Tipo de unin de esta condicin de bsqueda con otras condiciones
        /// </summary>
        public FilterUnion ExternalUnion
        {
            get { return externalUnion; }
            set { externalUnion = value; }
        }

        /// <summary>
        /// Tipo de unin de esta condicin de bsqueda con otras condiciones
        /// </summary>
        public FilterUnion DefaultFiltersUnion
        {
            get { return defaultFiltersUnion; }
            set { defaultFiltersUnion = value; }
        }


        /// <summary>
        /// Devuelve la expresin de texto de esta condicin de bsqueda
        /// </summary>
        /// <returns>String con el comando SQL</returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        /// <summary>
        /// Clase que deben implementar las condiciones de bsqueda
        /// y que devolvern su representacin de texto del
        /// comando SQL
        /// </summary>
        /// <returns></returns>
        public abstract string GetPredicateExpression();

        #region Miembros de ICloneable

        /// <summary>
        /// Todas las condiciones de bsqueda pueden 
        /// </summary>
        /// <returns></returns>
        public abstract object Clone();

        #endregion
    }
}