using System;
using System.Collections.Generic;
using System.Data;
using System.Linq.Expressions;

namespace Avanzis.QueryBuilder.Fluent
{
    ///<summary>
    ///</summary>
    ///<typeparam name = "T"></typeparam>
    public class Select<T> : ISelect, ICloneable where T : class
    {
        protected SelectBuilder builder;

        #region Constructor

        ///<summary>
        ///</summary>
        public Select()
        {
            this.builder = new SelectBuilder();
        }

        ///<summary>
        ///</summary>
        ///<param name = "builder"></param>
        public Select(SelectBuilder builder)
        {
            this.builder = (SelectBuilder) builder.Clone();
        }

        ///<summary>
        ///</summary>
        ///<param name = "select"></param>
        public Select(Select select)
        {
            this.builder = select.ToSelectBuilder();
        }

        ///<summary>
        ///</summary>
        ///<param name = "select"></param>
        public Select(Select<T> select)
        {
            this.builder = select.ToSelectBuilder();
        }

        #endregion

        #region Field

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> Field(string expression)
        {
            builder.Fields.Add(expression);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "alias"></param>
        ///<returns></returns>
        public virtual Select<T> Field(string expression, string alias)
        {
            builder.Fields.Add(expression, alias);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> Field(Expression<Func<T, object>> expression)
        {
            var stringExpression = FluentHelper.ParseProperty(expression);
            builder.Fields.Add(stringExpression);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "propertyExpression"></param>
        ///<returns></returns>
        public virtual Select<T> Field(string expression, Expression<Func<T, object>> propertyExpression)
        {
            var alias = FluentHelper.ParseProperty(propertyExpression);
            builder.Fields.Add(expression, alias);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Field<K>(Expression<Func<K, object>> expression)
        {
            SqlField field = FluentHelper.GetField(expression);
            builder.Fields.Add(field);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "propertyExpression"></param>
        ///<param name = "aliasExpression"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Field<K>(Expression<Func<K, object>> propertyExpression, Expression<Func<T, object>> aliasExpression)
        {
            SqlField field = FluentHelper.GetField(propertyExpression, aliasExpression);
            builder.Fields.Add(field);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "propertyExpression"></param>
        ///<param name = "aliasExpression"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Field<K>(string propertyExpression, Expression<Func<T, object>> aliasExpression)
        {
            SqlField field = FluentHelper.GetField<K, T>(propertyExpression, aliasExpression);
            builder.Fields.Add(field);
            return this;
        }


        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Field<K>(string expression)
        {
            SqlField field = FluentHelper.GetField<K>(expression);
            builder.Fields.Add(field);
            return this;
        }

        #endregion

        #region From
        
        ///<summary>
        ///</summary>
        ///<returns></returns>
        public virtual Select<T> From()
        {
            string tableName = FluentHelper.GetTableName<T>();
            builder.From.TableName = tableName;
            return this;
        }

        ///<summary>
        ///</summary>
        ///<returns></returns>
        public virtual Select<T> From(string tableName)
        {
            builder.From.TableName = tableName;
            return this;
        }

        ///<summary>
        ///</summary>
        ///<typeparam name="K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> From<K>()
        {
            string tableName = FluentHelper.GetTableName<K>();
            builder.From.TableName = tableName;
            return this;
        }

        #endregion

        #region Where
        
        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Where(Expression<Func<T, object>> expression, object fieldValue)
        {
            var stringExpression = FluentHelper.ParseProperty(expression);
            if (!(fieldValue is SqlOperator))
            {
                builder.Where.Filters.Add(stringExpression, fieldValue);
            }
            else
            {
                builder.Where.Filters.Add(stringExpression, (SqlOperator) fieldValue, null);
            }
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name="sqlOperator"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Where(Expression<Func<T, object>> expression, SqlOperator sqlOperator, object fieldValue)
        {
            var stringExpression = FluentHelper.ParseProperty(expression);
            builder.Where.Filters.Add(stringExpression, sqlOperator, fieldValue);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Where<K>(Expression<Func<K, object>> expression, object fieldValue)
        {
            return Where(expression, SqlOperator.Equals, fieldValue, ExpressionType.Primitive);
        }

        ///<summary>
        ///</summary>
        ///<param name = "propertyExpression"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Where<K>(string propertyExpression, object fieldValue)
        {
            return Where<K>(propertyExpression, fieldValue, ExpressionType.Primitive);
        }

        ///<summary>
        ///</summary>
        ///<param name = "propertyExpression"></param>
        ///<param name = "fieldValue"></param>
        ///<param name="expressionType"></param>
        ///<returns></returns>
        public virtual Select<T> Where<K>(string propertyExpression, object fieldValue, ExpressionType expressionType)
        {
            SqlField sqlField = FluentHelper.GetField<K, T>(propertyExpression);
            var stringExpression = sqlField.ToString();
            builder.Where.Filters.Add(stringExpression, SqlOperator.Equals, fieldValue, expressionType);
            return this;
        }


        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "compareOperator"></param>
        ///<param name = "fieldValue"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Where<K>(Expression<Func<K, object>> expression, SqlOperator compareOperator, object fieldValue)
        {
            return Where<K>(expression, compareOperator, fieldValue, ExpressionType.Primitive);
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "compareOperator"></param>
        ///<param name = "fieldValue"></param>
        ///<param name="expressionType"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Where<K>(Expression<Func<K, object>> expression, SqlOperator compareOperator, object fieldValue, ExpressionType expressionType)
        {
            SqlField sqlField = FluentHelper.GetField(expression);
            var stringExpression = sqlField.ToString();
            builder.Where.Filters.Add(stringExpression, compareOperator, fieldValue, expressionType);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "leftProperty"></param>
        ///<param name = "compareOperator"></param>
        ///<param name = "rightProperty"></param>
        ///<returns></returns>
        public virtual Select<T> Where<T1, T2>(Expression<Func<T1, object>> leftProperty, SqlOperator compareOperator, Expression<Func<T2, object>> rightProperty)
        {
            SqlField leftField = FluentHelper.GetField(leftProperty);
            var leftStringExpression = leftField.ToString();
            SqlField rightField = FluentHelper.GetField(rightProperty);
            var rightStringExpression = rightField.ToString();
            builder.Where.Filters.Add(leftStringExpression, compareOperator, rightStringExpression, ExpressionType.Primitive);
            return this;
        }


        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Where(string fieldName, object fieldValue)
        {
            builder.Where.Filters.Add(fieldName, fieldValue);
            return this;
        }

        /// <summary>
        /// </summary>
        /// <param name = "fieldName"></param>
        /// <param name = "compareOperator"></param>
        /// <param name = "fieldValue"></param>
        /// <returns></returns>
        public virtual Select<T> Where(string fieldName, SqlOperator compareOperator, object fieldValue)
        {
            builder.Where.Filters.Add(fieldName, compareOperator, fieldValue);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "filters"></param>
        ///<returns></returns>
        public virtual Select<T> Where(Filters filters)
        {
            var filterExpression = filters.ToString();
            builder.Where.Filters.Add(filterExpression);
            return this;
        }

        #endregion

        #region Join

        ///<summary>
        ///</summary>
        ///<param name = "leftExpression"></param>
        ///<param name = "rightExpression"></param>
        ///<returns></returns>
        public virtual Select<T> Join<K>(Expression<Func<T, object>> leftExpression, Expression<Func<K, object>> rightExpression) where K : class
        {
            string tableSource = FluentHelper.GetTableName<K>();
            string joinExpression = FluentHelper.GetTableName<T>() + "." + FluentHelper.ParseProperty(leftExpression) + " = " + tableSource + "." + FluentHelper.ParseProperty(rightExpression);

            builder.Joins.Add(tableSource, joinExpression);

            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "leftExpression"></param>
        ///<param name = "rightExpression"></param>
        ///<returns></returns>
        public virtual Select<T> Join<K1, K2>(Expression<Func<K1, object>> leftExpression, Expression<Func<K2, object>> rightExpression) where K1 : class where K2 : class
        {
            string tableSource = FluentHelper.GetTableName<K2>();
            SqlField leftField = FluentHelper.GetField(leftExpression);
            SqlField rightField = FluentHelper.GetField(rightExpression);

            builder.Joins.Add(JoinTypes.Inner, tableSource, leftField.ToString(), SqlOperator.Equals, rightField.ToString());

            return this;
        }

        #endregion

        #region LeftJoin

        ///<summary>
        ///</summary>
        ///<param name = "leftExpression"></param>
        ///<param name = "rightExpression"></param>
        ///<returns></returns>
        public virtual Select<T> LeftJoin<K1, K2>(Expression<Func<K1, object>> leftExpression, Expression<Func<K2, object>> rightExpression)
            where K1 : class
            where K2 : class
        {
            string tableSource = FluentHelper.GetTableName<K2>();
            SqlField leftField = FluentHelper.GetField(leftExpression);
            SqlField rightField = FluentHelper.GetField(rightExpression);

            builder.Joins.Add(JoinTypes.Left, tableSource, leftField.ToString(), SqlOperator.Equals, rightField.ToString());

            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "leftExpression"></param>
        ///<param name = "rightExpression"></param>
        ///<param name = "tableAlias"></param>
        ///<returns></returns>
        public virtual Select<T> LeftJoin<K1, K2>(Expression<Func<K1, object>> leftExpression, Expression<Func<K2, object>> rightExpression, string tableAlias)
            where K1 : class
            where K2 : class
        {
            string tableSource = FluentHelper.GetTableName<K2>();
            SqlField leftField = FluentHelper.GetField(leftExpression);
            SqlField rightField = FluentHelper.GetField(rightExpression);

            // Replaces right field name for alias
            rightField.TableName = tableAlias;

            builder.Joins.Add(JoinTypes.Left, tableSource, tableAlias, leftField.ToString(), SqlOperator.Equals, rightField.ToString());

            return this;
        }

        #endregion

        #region Distinct

        /// <summary>
        /// Return distinct fields
        /// </summary>
        /// <returns></returns>
        public Select<T> Distinct()
        {
            builder.Select.SelectionType = SelectionTypes.Distinct;
            return this;
        }

        #endregion

        #region GroupBy

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> GroupBy(Expression<Func<T, object>> expression)
        {
            SqlField sqlField = FluentHelper.GetField(expression, false);
            builder.GroupBy.Fields.Add(sqlField);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> GroupBy<K1>(Expression<Func<K1, object>> expression)
        {
            SqlField sqlField = FluentHelper.GetField(expression);
            builder.GroupBy.Fields.Add(sqlField);
            return this;
        }

        #endregion

        #region Having

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> Having(string expression)
        {
            builder.Having.Filters.Add(expression);
            return this;
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="expression"></param>
        /// <param name="fieldValue"></param>
        /// <returns></returns>
        public virtual Select<T> Having(Expression<Func<T, object>> expression, object fieldValue)
        {
            SqlField sqlField = FluentHelper.GetField(expression, false);
            var stringExpression = sqlField.ToString();
            builder.Having.Filters.Add(stringExpression, SqlOperator.Equals, fieldValue, ExpressionType.Primitive);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "compareOperator"></param>
        ///<param name = "fieldValue"></param>
        ///<param name="expressionType"></param>
        ///<typeparam name = "K"></typeparam>
        ///<returns></returns>
        public virtual Select<T> Having<K>(Expression<Func<K, object>> expression, SqlOperator compareOperator, object fieldValue, ExpressionType expressionType)
        {
            SqlField sqlField = FluentHelper.GetField(expression);
            var stringExpression = sqlField.ToString();
            builder.Having.Filters.Add(stringExpression, compareOperator, fieldValue, expressionType);
            return this;
        }


        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Having<TK>(Expression<Func<TK, object>> fieldName, object fieldValue)
        {
            var stringExpression = FluentHelper.ParseProperty(fieldName);

            builder.Having.Filters.Add(stringExpression, fieldValue);

            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "compareOperator"></param>
        ///<param name = "fieldValue"></param>
        ///<returns></returns>
        public virtual Select<T> Having<TK>(Expression<Func<TK, object>> fieldName, SqlOperator compareOperator, object fieldValue)
        {
            var stringExpression = FluentHelper.ParseProperty(fieldName);

            builder.Having.Filters.Add(stringExpression, compareOperator, fieldValue);

            return this;
        }

        /// <summary>
        /// </summary>
        /// <param name = "fieldName"></param>
        /// <param name = "fieldValue"></param>
        /// <returns></returns>
        public virtual Select<T> Having(string fieldName, object fieldValue)
        {
            builder.Having.Filters.Add(fieldName, fieldValue);

            return this;
        }

        /// <summary>
        /// </summary>
        /// <param name = "fieldName"></param>
        /// <param name = "compareOperator"></param>
        /// <param name = "fieldValue"></param>
        /// <returns></returns>
        public virtual Select<T> Having(string fieldName, SqlOperator compareOperator, object fieldValue)
        {
            builder.Having.Filters.Add(fieldName, compareOperator, fieldValue);

            return this;
        }

        #endregion

        #region OrderByDescending

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderByDescending(string fieldName)
        {
            return OrderBy(fieldName, OrderDirection.Desc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderByDescending(Expression<Func<T, object>> fieldName)
        {
            return OrderBy(fieldName, OrderDirection.Desc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> OrderByDescending<K>(Expression<Func<K, object>> expression)
        {
            return OrderBy(expression, OrderDirection.Desc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderByDescending<K>(string fieldName)
        {
            return OrderBy<K>(fieldName, OrderDirection.Desc);
        }

        #endregion

        #region OrderByAscending

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy(string fieldName)
        {
            return OrderBy(fieldName, OrderDirection.Asc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy(Expression<Func<T, object>> fieldName)
        {
            return OrderBy(fieldName, OrderDirection.Asc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy<K>(Expression<Func<K, object>> expression)
        {
            return OrderBy(expression, OrderDirection.Asc);
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy<K>(string fieldName)
        {
            return OrderBy<K>(fieldName, OrderDirection.Asc);
        }

        #endregion

        #region OrderByWithDirection

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "direction"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy(string fieldName, OrderDirection direction)
        {
            builder.OrderBy.Fields.Add(fieldName, direction);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "direction"></param>
        ///<returns></returns>
        public virtual Select<T> OrderBy(Expression<Func<T, object>> fieldName, OrderDirection direction)
        {
            SqlField sqlField = FluentHelper.GetField(fieldName, false);
            builder.OrderBy.Fields.Add(sqlField, direction);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "expression"></param>
        ///<param name = "direction"></param>
        ///<returns></returns>
        private Select<T> OrderBy<K>(Expression<Func<K, object>> expression, OrderDirection direction)
        {
            SqlField sqlField = FluentHelper.GetField(expression);
            builder.OrderBy.Fields.Add(sqlField, direction);
            return this;
        }

        ///<summary>
        ///</summary>
        ///<param name = "fieldName"></param>
        ///<param name = "direction"></param>
        ///<returns></returns>
        private Select<T> OrderBy<K>(string fieldName, OrderDirection direction)
        {
            SqlField sqlField = new SqlField();
            sqlField.ColumnNameOrExpression = fieldName;
            sqlField.TableName = FluentHelper.GetTableName<K>();
            builder.OrderBy.Fields.Add(sqlField, direction);
            return this;
        }

        #endregion

        #region Take

        ///<summary>
        ///</summary>
        ///<param name = "limit"></param>
        ///<returns></returns>
        public virtual Select<T> Take(int limit)
        {
            builder.Limit = limit;
            return this;
        }

        #endregion

        #region Skip

        ///<summary>
        ///</summary>
        ///<param name = "offset"></param>
        ///<returns></returns>
        public virtual Select<T> Skip(int offset)
        {
            builder.Offset = offset;
            return this;
        }

        #endregion

        #region ToString

        public override string ToString()
        {
            // Aade el from por defecto
            if (builder.From.TableName == null)
                From();

            return builder.ToString();
        }

        #endregion

        #region Clone

        /// <summary>
        ///   Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        ///   A new object that is a copy of this instance.
        /// </returns>
        /// <filterpriority>2</filterpriority>
        public virtual object Clone()
        {
            var select = new Select<T>((SelectBuilder) builder.Clone());
            return select;
        }

        #endregion

        #region ToSelectBuilder

        ///<summary>
        ///</summary>
        ///<returns></returns>
        public SelectBuilder ToSelectBuilder()
        {
            return (SelectBuilder) builder.Clone();
        }

        #endregion

        #region Fields

        ///<summary>
        ///</summary>
        ///<param name = "fields"></param>
        ///<returns></returns>
        ///<exception cref = "NotImplementedException"></exception>
        public virtual Select<T> Fields(Fields<T> fields)
        {
            builder.Fields.AddRange(fields);
            return this;
        }


        #endregion

        
    }
}