using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clausula FROM.
    /// </summary>	
    public class FromClause : ICloneable
    {
        private TableSource mainTableSource;
        private JoinTableCollection tableSources;

        #region Properties

        /// <summary>
        /// Nombre de la tabla, o expresin, de la que se cargarn los datos
        /// </summary>
        public string TableName
        {
            get { return mainTableSource.TableName; }
            set { mainTableSource.TableName = value; }
        }

        /// <summary>
        /// Alias que tomar la expresin del FROM
        /// </summary>
        public string Alias
        {
            get { return mainTableSource.Alias; }
            set { mainTableSource.Alias = value; }
        }

        /// <summary>
        /// Gets the main table source.
        /// </summary>
        /// <value>The main table source.</value>
        internal TableSource MainTableSource
        {
            get { return mainTableSource; }
        }

        /// <summary>
        /// Coleccin de Joins con otras tablas o expresiones
        /// </summary>
        public JoinTableCollection Joins
        {
            get
            {
                if (tableSources == null)
                    tableSources = new JoinTableCollection();
                return tableSources;
            }
        }

        #endregion

        #region Constructor

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla principal o expresin</param>
        /// <param name="alias">Alias para esa tabla o expresin</param>
        public FromClause(string tableName, string alias)
        {
            mainTableSource = new TableSource(tableName, alias);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla principal o expresin</param>
        public FromClause(string tableName)
        {
            mainTableSource = new TableSource(tableName);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public FromClause()
        {
            mainTableSource = new TableSource();
        }

        #endregion

        /// <summary>
        /// Representacin SQL de la clasula FROM
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            if (String.IsNullOrEmpty(mainTableSource.TableName))
                throw new ArgumentNullException("Table cannot be null", "TableName");

            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona una clusula From
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            FromClause cloned = new FromClause();
            cloned.tableSources = (JoinTableCollection) this.Joins.Clone();
            cloned.mainTableSource = (TableSource) this.mainTableSource.Clone();
            return cloned;
        }

        #endregion
    }
}