using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Tabla de Join
    /// </summary>
    public class JoinTable : TableSource, ICloneable
    {
        private JoinTypes joinType = JoinTypes.Inner;
        private SearchCondition searchCondition;
        private TableSource tableSource;

        /// <summary>
        /// Constructor
        /// </summary>
        public JoinTable() : this(JoinTypes.Inner, null, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableSource"></param>
        public JoinTable(string tableSource) : this(JoinTypes.Inner, tableSource, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="joinType"></param>
        public JoinTable(JoinTypes joinType) : this(joinType, null, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="joinType"></param>
        /// <param name="tableSource"></param>
        public JoinTable(JoinTypes joinType, string tableSource) : this(joinType, tableSource, null)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableSource"></param>
        /// <param name="joinCondition"></param>
        public JoinTable(string tableSource, string joinCondition) : this(JoinTypes.Inner, tableSource, joinCondition)
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="joinType"></param>
        /// <param name="tableSource"></param>
        /// <param name="joinCondition"></param>
        public JoinTable(JoinTypes joinType, string tableSource, string joinCondition)
        {
            this.joinType = joinType;
            if (tableSource != null)
                this.TableSource = new TableSource(tableSource);
            if (joinCondition != null)
                this.Filters.Add(joinCondition);
        }

        /// <summary>
        /// Tipo de JoinQue se realizar
        /// </summary>
        public JoinTypes JoinType
        {
            get { return joinType; }
            set { joinType = value; }
        }

        /// <summary>
        /// Origen de la tabla
        /// </summary>
        public TableSource TableSource
        {
            get { return tableSource ?? (tableSource = new TableSource()); }
            set { tableSource = value; }
        }

        /// <summary>
        /// Nombre de la tabla
        /// </summary>
        public override string TableName
        {
            get { return TableSource.TableName; }
            set { TableSource.TableName = value; }
        }
        

        /// <summary>
        /// Condicin para realizar el Join
        /// </summary>
        public SearchCondition SearchCondition
        {
            get
            {
                if (searchCondition == null)
                {
                    searchCondition = new SearchCondition();
                    searchCondition.Filters.DefaultExpressionType = ExpressionType.Complex;
                }
                return searchCondition;
            }
            set { searchCondition = value; }
        }

        /// <summary>
        /// Filtros que se utilizan para realizar el Join
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get { return SearchCondition.Filters; }
        }

        /// <summary>
        /// Tipo de unin que se utiliza para los filtros
        /// </summary>
        public FilterUnion Union
        {
            get { return SearchCondition.ExternalUnion; }
            set { SearchCondition.ExternalUnion = value; }
        }

        /// <summary>
        /// Obtiene la expresin de la tabla origen
        /// </summary>
        /// <returns></returns>
        public override string GetTableSourceExpression()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona una clusula Join
        /// </summary>
        /// <returns></returns>
        public new object Clone()
        {
            JoinTable cloned = new JoinTable();
            cloned.joinType = this.joinType;
            cloned.tableSource = (TableSource) this.tableSource.Clone();
            cloned.searchCondition = (SearchCondition) this.searchCondition.Clone();
            return cloned;
        }

        #endregion
    }
}