using System;
using System.Collections.ObjectModel;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clase de la coleccin de campos
    /// </summary>
    public class JoinTableCollection : Collection<JoinTable>, ICloneable
    {
        /// <summary>
        /// Aade un nuevo Join a una cierta tabla y con una condicin de bsqueda
        /// </summary>
        /// <param name="tableName"></param>
        /// <param name="joinCondition"></param>
        /// <returns></returns>
        public JoinTable Add(string tableName, string joinCondition)
        {
            JoinTable jt = new JoinTable(tableName, joinCondition);
            Add(jt);
            return jt;
        }

        /// <summary>
        /// Aade un nuevo Join a partir del tipo de unin, el origen de la tabla y la condicin de unin
        /// </summary>
        /// <param name="joinType"></param>
        /// <param name="tableSource"></param>
        /// <param name="joinCondition"></param>
        /// <returns></returns>
        public JoinTable Add(JoinTypes joinType, string tableSource, string joinCondition)
        {
            JoinTable jt = new JoinTable(joinType, tableSource, joinCondition);
            Add(jt);
            return jt;
        }

        /// <summary>
        /// Aade un nuevo Join a partir del origen de la tabla y las condicin de la unin
        /// </summary>
        /// <param name="tableSource"></param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public JoinTable Add(string tableSource, object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            JoinTable jt = new JoinTable(JoinTypes.Inner, tableSource);
            jt.Filters.Add(new SearchFilter(leftExpression, compareOperator, rightExpression, ExpressionType.Complex));
            Add(jt);
            return jt;
        }

        /// <summary>
        /// Aade un nuevo Join a partir del tipo de unin, origen de la tabla y las condicin de la unin
        /// </summary>
        /// <param name="joinType"></param>
        /// <param name="tableSource"></param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public JoinTable Add(JoinTypes joinType, string tableSource, object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            JoinTable jt = new JoinTable(joinType, tableSource);
            jt.Filters.Add(new SearchFilter(leftExpression, compareOperator, rightExpression, ExpressionType.Complex));
            Add(jt);
            return jt;
        }

        /// <summary>
        /// Aade un nuevo Join a partir del tipo de unin, origen de la tabla y las condicin de la unin
        /// </summary>
        /// <param name="joinType"></param>
        /// <param name="tableSource"></param>
        /// <param name="tableSourceAlias"></param>
        /// <param name="leftExpression"></param>
        /// <param name="compareOperator"></param>
        /// <param name="rightExpression"></param>
        /// <returns></returns>
        public JoinTable Add(JoinTypes joinType, string tableSource, string tableSourceAlias, object leftExpression, SqlOperator compareOperator, object rightExpression)
        {
            TableSource ts = new TableSource(tableSource, tableSourceAlias);
            JoinTable jt = new JoinTable(joinType, ts.ToString());
            jt.Filters.Add(new SearchFilter(leftExpression, compareOperator, rightExpression, ExpressionType.Complex));
            Add(jt);
            return jt;
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona la coleccin de Join
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            JoinTableCollection cloned = new JoinTableCollection();
            foreach (JoinTable joinTable in this.Items)
            {
                cloned.Add((JoinTable) joinTable.Clone());
            }

            return cloned;
        }

        #endregion
    }
}