using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Origenes de datos para Joins
    /// </summary>
    public class TableSource : ICloneable
    {
        private string alias;
        private string tableName;
        private bool useAlias = true;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        /// <param name="alias">Alias de la tabla</param>
        public TableSource(string tableName, string alias)
        {
            this.TableName = tableName;
            this.Alias = alias;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        /// <remarks>Analiza la expresin para obtener el nombre de la tabla y el alias</remarks>
        public TableSource(string tableName)
        {
            if (tableName == null)
                throw new ArgumentNullException("tableName");

            QueryProcessor.Current.Parse(this, tableName);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public TableSource(bool useAlias)
        {
            this.useAlias = useAlias;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        public TableSource()
        {
        }

        /// <summary>
        /// Nombre de la tabla de este origen de datos
        /// </summary>
        public virtual string TableName
        {
            get { return tableName; }
            set { tableName = value; }
        }

        /// <summary>
        /// Alias para este origen de datos
        /// </summary>
        public string Alias
        {
            get { return alias; }
            set { alias = value; }
        }

        /// <summary>
        /// Gets a value indicating whether [use alias].
        /// </summary>
        /// <value><c>true</c> if [use alias]; otherwise, <c>false</c>.</value>
        internal bool UseAlias
        {
            get { return useAlias; }
        }


        /// <summary>
        /// Obtiene la representacin SQL
        /// </summary>
        /// <returns></returns>
        public virtual string GetTableSourceExpression()
        {
            if (tableName == null)
                throw new ArgumentNullException("TableName cannot be empty", "TableName");

            return QueryProcessor.Current.Process(this);
        }

        /// <summary>
        /// Obtiene la representacin SQL
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return GetTableSourceExpression();
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de este origen de datos
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            TableSource cloned = new TableSource(this.TableName, this.Alias);
            return cloned;
        }

        #endregion
    }
}