using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clasula Group By
    /// </summary>
    /// <remarks>La clasula GroupBy se utiliza cuando una consulta
    /// tiene columnas con funciones de agregado</remarks>
    public class GroupByClause : ICloneable
    {
        private SqlFieldCollection groupByColumns;

        /// <summary>
        /// Constructor
        /// </summary>
        public GroupByClause()
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="groupByExpression">Expresin que se utilizar para agrupar</param>
        public GroupByClause(string groupByExpression)
        {
            Fields.AddRange(groupByExpression);
        }

        /// <summary>
        /// Columnas por los que se agrupar la consulta
        /// </summary>
        public SqlFieldCollection Fields
        {
            get
            {
                if (groupByColumns == null)
                    groupByColumns = new SqlFieldCollection();
                return groupByColumns;
            }
        }

        /// <summary>
        /// Devuelve la representacin en cadena de texto
        /// </summary>
        /// <returns>string con el texto SQL que se utilizar</returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon del groupby actual
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            GroupByClause cloned = new GroupByClause();
            cloned.groupByColumns = (SqlFieldCollection) this.Fields.Clone();
            return cloned;
        }

        #endregion
    }
}