using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clusula HAVING
    /// </summary>
    public class HavingClause : ICloneable
    {
        private SearchCondition searchCondition;

        /// <summary>
        /// Condicin de bsqueda de la clusula Having
        /// </summary>
        public SearchCondition SearchCondition
        {
            get
            {
                if (searchCondition == null)
                {
                    searchCondition = new SearchCondition();
                    searchCondition.Filters.DefaultExpressionType = ExpressionType.Complex;
                }
                return searchCondition;
            }
            set { searchCondition = value; }
        }

        /// <summary>
        /// Coleccin de filtros utilizados en la condicin de bsqueda
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get { return SearchCondition.Filters; }
        }

        /// <summary>
        /// Tipo de unin que se utilizar para unir los filtros
        /// </summary>
        public FilterUnion Union
        {
            get { return SearchCondition.DefaultFiltersUnion; }
            set { SearchCondition.DefaultFiltersUnion = value; }
        }

        /// <summary>
        /// Si debe negar la condicin de bsqueda
        /// </summary>
        public bool Negative
        {
            get { return SearchCondition.Negative; }
            set { SearchCondition.Negative = value; }
        }

        /// <summary>
        /// Devuelve la instruccin SQL de la clasula HAVING
        /// </summary>
        /// <returns>string con la clasula SQL</returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Clona el la clasula Having
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            HavingClause cloned = new HavingClause();
            cloned.searchCondition = (SearchCondition) this.SearchCondition.Clone();
            return cloned;
        }

        #endregion
    }
}