using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clases para gener consultas INSERT
    /// </summary>
    public class InsertBuilder : SaveBuilder, ICloneable
    {
        private InsertClause insert;

        /// <summary>
        /// Constructor
        /// </summary>
        public InsertBuilder()
        {
        }

        /// <summary>
        /// Contructor
        /// </summary>
        /// <param name="tableName">Tabla donde se realizar la insercin</param>
        public InsertBuilder(string tableName)
        {
            this.Insert = new InsertClause(tableName);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        /// <param name="fieldValues">Coleccin de campos y valores</param>
        public InsertBuilder(string tableName, SqlFieldValueCollection fieldValues)
        {
            this.Insert = new InsertClause(tableName, fieldValues);
        }

        /// <summary>
        /// Clasula Insert
        /// </summary>
        public InsertClause Insert
        {
            get
            {
                if (insert == null)
                    insert = new InsertClause();
                return insert;
            }
            set { insert = value; }
        }

        /// <summary>
        /// Coleccin de campos y valores de los campos
        /// </summary>
        public override SqlFieldValueCollection FieldValues
        {
            get { return this.Insert.FieldValues; }
        }

        /// <summary>
        /// Nombre de la tabla
        /// </summary>
        public override string TableName
        {
            get { return this.Insert.TableName; }
            set { this.Insert.TableName = value; }
        }

        /// <summary>
        /// Aade un campo con su valor
        /// </summary>
        /// <param name="field">Campo</param>
        /// <param name="value">The value.</param>
        public void AddFieldValue(string field, object value)
        {
            this.Insert.FieldValues.Add(field, value);
        }

        /// <summary>
        /// Aade un campo con su valor
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="value">The value.</param>
        /// <param name="expressionType"> </param>
        public void AddFieldValue(string field, object value, ExpressionType expressionType)
        {
            this.Insert.FieldValues.Add(field, value, expressionType);
        }

        /// <summary>
        /// Establece el valor de un campo
        /// </summary>
        /// <param name="field">Campo</param>
        /// <param name="value">The value.</param>
        public void SetFieldValue(string field, object value)
        {
            if (this.Insert.FieldValues.ContainsKey(field))
                this.Insert.FieldValues[field] = value;
            else
                this.AddFieldValue(field, value);
        }

        /// <summary>
        /// Devuelve la representacin SQL del INSERT
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon del InsertBuilder actual
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            InsertBuilder cloned = new InsertBuilder();

            cloned.insert = (InsertClause) this.Insert.Clone();

            return cloned;
        }

        #endregion
    }
}