using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clusula INSERT
    /// </summary>
    public class InsertClause : ICloneable
    {
        private SqlFieldValueCollection fieldValues;
        private InsertTypes insertType = InsertTypes.Unknown;


        /// <summary>
        /// Tabla de origen
        /// </summary>
        private TableSource tableSource = new TableSource(false);

        /// <summary>
        /// Constructor
        /// </summary>
        public InsertClause()
        {
            //throw new NotImplementedException("Esta clusula no est implementada ni probada");
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        public InsertClause(string tableName)
        {
            this.TableName = tableName;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        /// <param name="fieldValues">Campos con sus valores que se van a actualizar</param>
        public InsertClause(string tableName, SqlFieldValueCollection fieldValues)
        {
            this.TableName = tableName;
            this.fieldValues = fieldValues;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="fieldValues">Campos con sus valores que se van a actualizar</param>
        public InsertClause(SqlFieldValueCollection fieldValues)
        {
            this.fieldValues = fieldValues;
        }

        /// <summary>
        /// Mtodo de insercin usado
        /// </summary>
        public InsertTypes InsertType
        {
            get { return insertType; }
            set { insertType = value; }
        }

        /// <summary>
        /// Gets or sets the table source.
        /// </summary>
        /// <value>The table source.</value>
        internal TableSource TableSource
        {
            get { return tableSource; }
            set { tableSource = value; }
        }

        /// <summary>
        /// Nombre de la tabla, o expresin, de la que se cargarn los datos
        /// </summary>
        public string TableName
        {
            get { return tableSource.TableName; }
            set { tableSource.TableName = value; }
        }

        /// <summary>
        /// Coleccin de campos y valores que se insertarn
        /// </summary>
        public SqlFieldValueCollection FieldValues
        {
            get
            {
                if (fieldValues == null)
                    fieldValues = new SqlFieldValueCollection();
                return fieldValues;
            }
        }

        /// <summary>
        /// EscapeValues = true;
        /// TODO: Faltaria implementar el caso para cuando no hay campos, solo valores.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            ExpressionType expressionType = ExpressionType.Primitive;

            if (this.InsertType == InsertTypes.OnlyFieldsAndParameters)
                expressionType = ExpressionType.Complex;

            return ToString(expressionType);
        }

        /// <summary>
        /// Converts to string
        /// </summary>
        /// <param name="expressionType">Type of the expression.</param>
        /// <returns></returns>
        public virtual string ToString(ExpressionType expressionType)
        {
            if (String.IsNullOrEmpty(TableName))
                throw new ArgumentException("Table name cannot be empty", "TableName");

            return QueryProcessor.Current.Process(this, expressionType);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la clusula actual
        /// </summary>
        /// <returns></returns>
        public virtual object Clone()
        {
            InsertClause clone = new InsertClause();
            clone.TableName = this.TableName;
            foreach (string field in this.FieldValues.Fields)
                clone.FieldValues.Add(field, this.FieldValues[field]);

            return clone;
        }

        #endregion
    }
}