using System;
using NUnit.Framework;
using Avanzis.Helpers.QueryBuilder;

namespace Avanzis.Helpers.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de FromClauseTest.
	/// </summary>
	[TestFixture]
	public class FromClauseTest
	{
		public FromClauseTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}

		[Test]
		public void SimpleFrom()
		{
			FromClause from = new FromClause();			
			from.TableName = "Customers";
			Assert.AreEqual("FROM Customers", from.ToString());			

			from = new FromClause("Customers");			
			Assert.AreEqual("FROM Customers", from.ToString());			
			
			from = new FromClause();			
			from.TableName = "Customers";
			from.Alias = "c";
			Assert.AreEqual("FROM Customers AS c", from.ToString());	
		
			from = new FromClause("Customers", "c");			
			Assert.AreEqual("FROM Customers AS c", from.ToString());
			
		}

		[Test]
		public void SimpleTableJoin()
		{
			FromClause from = new FromClause("Customers");		
			from.Joins.Add(JoinTypes.INNER, "Invoices", "Customers.CustomerId = Invoices.CustomerId");
			string result = "FROM Customers INNER JOIN Invoices ON ((Customers.CustomerId = Invoices.CustomerId))";
			Assert.AreEqual(result, from.ToString());

			from = new FromClause();		
			from.TableName = "Customers";
			from.Alias = "c";
			from.Joins.Add(JoinTypes.INNER, "Invoices", "Customers.CustomerId = Invoices.CustomerId");
			result = "FROM Customers AS c INNER JOIN Invoices ON ((Customers.CustomerId = Invoices.CustomerId))";
			Assert.AreEqual(result, from.ToString());

			from = new FromClause("Customers", "c");		
			from.Joins.Add(JoinTypes.INNER, "Invoices", "Customers.CustomerId = Invoices.CustomerId");
			result = "FROM Customers AS c INNER JOIN Invoices ON ((Customers.CustomerId = Invoices.CustomerId))";
			Assert.AreEqual(result, from.ToString());

			from = new FromClause("Customers");
			from.Joins.Add(JoinTypes.LEFT, "Invoices", "Customers.CustomerId", SqlOperator.Equals, "Invoices.CustomerId");
			result = "FROM Customers LEFT JOIN Invoices ON ((Customers.CustomerId = Invoices.CustomerId))";
			Assert.AreEqual(result, from.ToString());
		}

		[Test] 
		public void MultipleJoinConditions()
		{
			FromClause from = new FromClause("Articles");		
			JoinTable jt = from.Joins.Add("ArticleFamilies", "Articles.ArticleFamilyId = ArticleFamilies.ArticleFamilyId");			
			jt.Filters.Add("Articles.ArticleType = ArticleFamilies.ArticleType");

			string result = "FROM Articles " +
				"INNER JOIN ArticleFamilies ON " + 
				"((Articles.ArticleFamilyId = ArticleFamilies.ArticleFamilyId) " + 
				"OR (Articles.ArticleType = ArticleFamilies.ArticleType))";

			Assert.AreEqual(result, from.ToString());		

			jt.SearchCondition.Union = FilterUnion.AND;

			result = "FROM Articles " +
				"INNER JOIN ArticleFamilies ON " + 
				"((Articles.ArticleFamilyId = ArticleFamilies.ArticleFamilyId) " + 
				"AND (Articles.ArticleType = ArticleFamilies.ArticleType))";

			Assert.AreEqual(result, from.ToString());		

		}

		[Test]
		public void MultipleJoins()
		{
			FromClause from = new FromClause("Articles");		
			from.Joins.Add("ArticleFamilies", "Articles.ArticleFamilyId = ArticleFamilies.ArticleFamilyId");
			from.Joins.Add("Manufacturers", "Articles.ManufacturerId = Manufacturers.ManufacturerId");
			from.Joins.Add(JoinTypes.LEFT, "WarehouseMovements", "Articles.ArticleId = WarehouseMovements.ArticleId");

			string result = "FROM Articles " +
				"INNER JOIN ArticleFamilies ON ((Articles.ArticleFamilyId = ArticleFamilies.ArticleFamilyId)) " + 
				"INNER JOIN Manufacturers ON ((Articles.ManufacturerId = Manufacturers.ManufacturerId)) " + 
				"LEFT JOIN WarehouseMovements ON ((Articles.ArticleId = WarehouseMovements.ArticleId))";

			Assert.AreEqual(result, from.ToString());		
			
		}

		[Test]
		public void MultipleTableJoins()
		{		
			


		}

		[Test]
		public void SimpleTableSourceTest()
		{
			TableSource st = new TableSource("Customer");
			Assert.AreEqual("Customer", st.ToString());

			st = new TableSource("Customer", "c");
			Assert.AreEqual("Customer AS c", st.ToString());
		}

		[Test]
		public void JoinedTableSource()
		{			
			JoinTable jts = new JoinTable(JoinTypes.INNER, "Invoices", "Customers.CustomerId = Invoices.CustomerId");					
			// jts.Filters.Add(new SearchFilter("Customers.CustomerId", SqlOperator.Equals, "Invoices.CustomerId"));
			

			string result = "INNER JOIN Invoices ON ((Customers.CustomerId = Invoices.CustomerId))";
            Assert.AreEqual(result, jts.ToString());           			

		}


	}
}
