using System;
using NUnit.Framework;
using Avanzis.Helpers.QueryBuilder;

namespace Avanzis.Helpers.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de OrderClauseTest.
	/// </summary>
	[TestFixture]
	public class OrderClauseTest
	{
		public OrderClauseTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}

		[Test]
		public void TestOrderItem()
		{
			SqlField sqlField = new SqlField("Articles.Name AS ArticleName");
			OrderItem orderItem = new OrderItem(sqlField);
			Assert.AreEqual("Articles.Name ASC", orderItem.ToString());

			orderItem = new OrderItem(sqlField, "DESC");
			Assert.AreEqual(OrderDirection.DESC, orderItem.Direction);
			Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

			orderItem = new OrderItem(sqlField, OrderDirection.DESC);
			Assert.AreEqual(OrderDirection.DESC, orderItem.Direction);
			Assert.AreEqual("Articles.Name DESC", orderItem.ToString());

			orderItem = new OrderItem("SUM(Movements) > 23 DESC");
			Assert.AreEqual(OrderDirection.DESC, orderItem.Direction);
			Assert.AreEqual("SUM(Movements) > 23", orderItem.Field.ToString());
		}

		[Test]
		public void EmptyOrderBy()
		{
			OrderByClause orderBy = new OrderByClause();
			Assert.AreEqual("", orderBy.ToString());
		}

		[Test]
		public void SimpleOrderBy()
		{
			OrderByClause orderBy = new OrderByClause("Articles.Name DESC");
			Assert.AreEqual("ORDER BY Articles.Name DESC", orderBy.ToString());

			orderBy = new OrderByClause();
			orderBy.Fields.Add("Articles.Name DESC");
			Assert.AreEqual("ORDER BY Articles.Name DESC", orderBy.ToString());
		}

		[Test]
		public void MultipleOrderBy()
		{
			OrderByClause orderBy = new OrderByClause("Articles.Name DESC, Articles.Date ASC");
			Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Date ASC", orderBy.ToString());
			
			orderBy = new OrderByClause();
			orderBy.Fields.Add("Articles.Name DESC");
			orderBy.Fields.Add("Articles.Date ASC");
			Assert.AreEqual("ORDER BY Articles.Name DESC,Articles.Date ASC", orderBy.ToString());
		}
	}
}
