using System;
using NUnit.Framework;
using Avanzis.Helpers.QueryBuilder;

namespace Avanzis.Helpers.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de SearchConditionTest.
	/// </summary>
	[TestFixture] 
	public class SearchConditionTest
	{
		public SearchConditionTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}

		[Test]
		public void SimpleSearchCondition()
		{			
			SqlField customerId = new SqlField("CustomerId", null, "Customers");
			SqlOperator sqlOperator = SqlOperator.Equals;
			SqlField paymentId = new SqlField("Payment.CustomerId");				

			
			SearchFilter se = new SearchFilter(customerId, sqlOperator, paymentId);			
			Assert.AreEqual("Customers.CustomerId = Payment.CustomerId", se.ToString());
			
			SearchCondition sc = new SearchCondition();
			sc.Filters.Add(se);			
			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId)", sc.ToString());

			SearchFilter se2 = new SearchFilter("[Products].VendorId = Shop.VendorId");			
			se2.Union = FilterUnion.AND;					
			Assert.AreEqual("[Products].VendorId = Shop.VendorId", se2.ToString());

			sc.Filters.Add(se2);		
			
			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)", sc.ToString());

			SearchFilter se3 = new SearchFilter(new SqlField("CountryId", null, "Shops"), sqlOperator, new SqlField("CountryId", null, "Countries"));
			se3.Union = FilterUnion.OR;			

			Assert.AreEqual("Shops.CountryId = Countries.CountryId", se3.ToString());

			sc.Filters.Add(se3);

			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId) OR (Shops.CountryId = Countries.CountryId)", sc.ToString());		
			
		}

		[Test]
		public void SimpleNegativeSearchCondition()
		{
			SearchCondition sc = new SearchCondition();
			sc.Negative = true;
			sc.Filters.Add("CustomerType = 'A'");		
			Assert.AreEqual("NOT (CustomerType = 'A')", sc.ToString());


		}

		[Test]
		public void ComplexNegativeSearchCondition()
		{
			SearchCondition sc = new SearchCondition();
			sc.Filters.Add("CustomerId = 2");
			sc.Filters.Add("Amount", SqlOperator.Greater, 0).Negative = true;
			
			Assert.AreEqual("(CustomerId = 2) OR NOT (Amount > 0)", sc.ToString());

			sc.Union = FilterUnion.AND;

			Assert.AreEqual("(CustomerId = 2) AND NOT (Amount > 0)", sc.ToString());			
		}

		[Test]
		public void ComplexSearchCondition()
		{			
			SqlField customerId = new SqlField("CustomerId", null, "Customers");
			SqlOperator sqlOperator = SqlOperator.Equals;
			SqlField paymentId = new SqlField("Payment.CustomerId");		

			SearchFilter se1 = new SearchFilter(customerId, sqlOperator, paymentId);			
			se1.Negative = false;			

			SearchFilter se2 = new SearchFilter("[Products].VendorId = Shop.VendorId");			
			se2.Union = FilterUnion.AND;			

			SearchFilter se3 = new SearchFilter(new SqlField("CountryId", null, "Shops"), sqlOperator, new SqlField("CountryId", null, "Countries"));
			se3.Union = FilterUnion.OR;			

			SearchCondition sc = new SearchCondition();
			SearchCondition sc1 = new SearchCondition();
			SearchCondition sc2 = new SearchCondition();

			sc1.Filters.Add(se1);
			sc1.Filters.Add(se2);
			sc2.Filters.Add(se3);

			sc2.Union = FilterUnion.AND;

			sc.Filters.Add(sc1);
			sc.Filters.Add(sc2);

			Assert.AreEqual("(Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)", sc1.ToString());
			Assert.AreEqual("(Shops.CountryId = Countries.CountryId)", sc2.ToString());
			Assert.AreEqual("((Customers.CustomerId = Payment.CustomerId) AND ([Products].VendorId = Shop.VendorId)) AND ((Shops.CountryId = Countries.CountryId))", sc.ToString());
			
		}

		[Test]
		public void SearchConditionSimpleOperators()
		{
			SearchFilter sf = new SearchFilter("ArticleId", SqlOperator.Equals, 1);
			Assert.AreEqual("ArticleId = 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Distinct, 1);
			Assert.AreEqual("ArticleId != 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Greater, 1);
			Assert.AreEqual("ArticleId > 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.GreaterOrEqual, 1);
			Assert.AreEqual("ArticleId >= 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.Less, 1);
			Assert.AreEqual("ArticleId < 1", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.LessOrEqual, 1);
			Assert.AreEqual("ArticleId <= 1", sf.ToString());				


		}

		[Test]
		public void SearchConditionStringOperators()
		{
			SearchFilter sf = new SearchFilter("ArticleId", SqlOperator.Like, "Test%");
			Assert.AreEqual("ArticleId LIKE 'Test%'", sf.ToString());

			sf = new SearchFilter("ArticleId", SqlOperator.NotLike, "Test%");
			Assert.AreEqual("ArticleId NOT LIKE 'Test%'", sf.ToString());
		}

		[Test]
		public void SearchConditionComplexOperators()
		{
			SearchFilter sf = new SearchFilter(SqlOperator.Contains, "QuantityPerUnit, 'bottles'");
			Assert.AreEqual("CONTAINS (QuantityPerUnit, 'bottles')", sf.ToString());

			sf = new SearchFilter(SqlOperator.Exists, "SELECT NULL");
			Assert.AreEqual("EXISTS (SELECT NULL)", sf.ToString());

			sf = new SearchFilter("state", SqlOperator.In, "'CA','IN','MD'");
			Assert.AreEqual("state IN ('CA','IN','MD')", sf.ToString());

			sf = new SearchFilter("state", SqlOperator.NotIn, "'CA','IN','MD'");
			Assert.AreEqual("state NOT IN ('CA','IN','MD')", sf.ToString());
		}


	}
}
