using System;
using NUnit.Framework;
using Avanzis.Helpers.QueryBuilder;

namespace Avanzis.Helpers.QueryBuilder.NUnit
{
	/// <summary>
	/// Descripcin breve de SqlFieldTest.
	/// </summary>
	[TestFixture]
	public class SqlFieldTest
	{
		public SqlFieldTest()
		{
			//
			// TODO: agregar aqu la lgica del constructor
			//
		}

		[Test]
		public void SimpleField()
		{
			SqlField field = new SqlField("CustomerId");
			Assert.AreEqual("CustomerId", field.ToString());

			field = new SqlField("CustomerId", "Id");
			Assert.AreEqual("CustomerId AS Id", field.ToString());

			field = new SqlField("CustomerId", "Id", "Customers");			
			Assert.AreEqual("Customers.CustomerId AS Id", field.ToString());

			field = new SqlField("CustomerId", null, "Customers");
			Assert.AreEqual("Customers.CustomerId", field.ToString());																  

		}

		[Test]
		public void ComplexFieldAssigment()
		{
			// El nombre de la tabla y la columna
			SqlField field = new SqlField("Customers.CustomerId");
			Assert.AreEqual("Customers", field.TableName);
			Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
			Assert.AreEqual(null, field.Alias);

			// El nombre de la tabla en corchetes y la columna
			field = new SqlField("[Customers].CustomerId");
			Assert.AreEqual("[Customers]", field.TableName);
			Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
			Assert.AreEqual(null, field.Alias);

			// El nombre de la tabla y la columna en corchetes
			field = new SqlField("Customers.[CustomerId]");
			Assert.AreEqual("Customers", field.TableName);
			Assert.AreEqual("[CustomerId]", field.ColumnNameOrExpression);
			Assert.AreEqual(null, field.Alias);

			// El nombre de la tabla, la columna y el Alias
			field = new SqlField("Customers.CustomerId AS cid");
			Assert.AreEqual("Customers", field.TableName);
			Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
			Assert.AreEqual("cid", field.Alias);

			// El nombre de la tabla, la columna y el Alias entre comillas
			field = new SqlField("Customers.CustomerId AS 'cid'");
			Assert.AreEqual("Customers", field.TableName);
			Assert.AreEqual("CustomerId", field.ColumnNameOrExpression);
			Assert.AreEqual("'cid'", field.Alias);
           	
			// Una expresin
			field = new SqlField("SUM(Quantity)");
			Assert.AreEqual(null, field.TableName);
			Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
			Assert.AreEqual(null, field.Alias);

			// Una expresin con Alias
			field = new SqlField("SUM(Quantity) AS Suma");
			Assert.AreEqual(null, field.TableName);
			Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
			Assert.AreEqual("Suma", field.Alias);

			// Una expresin con Alias entre comillas simples
			field = new SqlField("SUM(Quantity) AS 'Suma Total'");
			Assert.AreEqual(null, field.TableName);
			Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
			Assert.AreEqual("'Suma Total'", field.Alias);

			// Una expresin con Alias y el AS sin que sea mayscula
			field = new SqlField("SUM(Quantity) aS Suma");
			Assert.AreEqual(null, field.TableName);
			Assert.AreEqual("SUM(Quantity)", field.ColumnNameOrExpression);
			Assert.AreEqual("Suma", field.Alias);
			
			// Una expresin entre parntesis con Alias
			field = new SqlField("(SELECT SUM(Quantity) AS QuantitySum FROM WarehouseMovements) AS Suma");
			Assert.AreEqual(null, field.TableName);
			Assert.AreEqual("(SELECT SUM(Quantity) AS QuantitySum FROM WarehouseMovements)", field.ColumnNameOrExpression);
			Assert.AreEqual("Suma", field.Alias);			
		}


		[Test]
		public void FieldCollection()
		{
			SqlFieldCollection fieldCollection = new SqlFieldCollection();
			fieldCollection.Add(new SqlField("CustomerId"));
			fieldCollection.Add(new SqlField("Name", "Customer Name"));
			fieldCollection.Add(new SqlField("SUM(Sales.Ammount)", "Sales"));
			Assert.AreEqual("CustomerId,Name AS 'Customer Name',SUM(Sales.Ammount) AS Sales", fieldCollection.ToString());
		}
		
	}
}
