using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Ordenacin
    /// </summary>
    public class OrderByClause : ICloneable
    {
        private OrderItemsCollection orderItems;

        /// <summary>
        /// Constructor
        /// </summary>
        public OrderByClause()
        {
            this.orderItems = new OrderItemsCollection();
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="orderExpression"></param>
        public OrderByClause(string orderExpression)
        {
            this.Fields.AddRange(orderExpression);
        }

        /// <summary>
        /// Cada una de las expresiones de ordenacin
        /// </summary>
        public OrderItemsCollection Fields
        {
            get
            {
                if (orderItems == null)
                    orderItems = new OrderItemsCollection();
                return orderItems;
            }
        }

        /// <summary>
        /// Traduccin a sql de la ordenacin
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la expresin de ordenacin
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            OrderByClause clone = new OrderByClause();
            clone.orderItems = (OrderItemsCollection) this.orderItems.Clone();
            return clone;
        }

        #endregion
    }
}