using System;
using System.Text;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Procesador para Microsoft Sql Server 2000. Convierte los objetos del QueryBuilder a sentencias SQL compatibles con SQL 2000
    /// </summary>
    internal class MsSql2000Processor : QueryProcessor
    {
        
        private string _parameterToken = "@";

        /// <summary>
        /// Gets the parameter symbol.
        /// </summary>
        /// <value>The parameter symbol.</value>
        public override string ParameterToken
        {
            get
            {
                return this._parameterToken;
            }
            set
            {
                this._parameterToken = value;
            }
        }

        public override string StartingQuoteToken
        {
            get { return "["; }
        }

        public override string EndingQuoteToken
        {
            get { return "]"; }
        }

        /// <summary>
        /// Processes the specified select clause.
        /// </summary>
        /// <param name="selectClause">The select clause.</param>
        /// <returns></returns>
        public override string Process(SelectClause selectClause)
        {
            if (selectClause == null)
                throw new ArgumentNullException("selectClause");

            if (selectClause.Offset > 0)
                throw new InvalidOperationException("Offset not supported by this dialect");

            if (selectClause.EnableCalculateFoundRows)
                throw new InvalidOperationException("Calculate found rows not supported by this dialect");

            return ProcessSelectClause(selectClause);
        }

        /// <summary>
        /// Processes the select clause.
        /// </summary>
        /// <param name="selectClause">The select clause.</param>
        /// <returns></returns>
        protected static string ProcessSelectClause(SelectClause selectClause)
        {
            StringBuilder sb = new StringBuilder();

            sb.Append("SELECT ");

            // Distinct
            if (selectClause.SelectionType == SelectionTypes.Distinct)
            {
                sb.Append("DISTINCT ");
            }

            // Limit
            if (selectClause.Limit >= 0)
            {
                sb.Append("TOP ");
                sb.Append(selectClause.Limit);
                sb.Append(" ");
            }

            // Fields
            if (selectClause.Fields.Count > 0)
                sb.Append(selectClause.Fields.ToString());
            else
                sb.Append("*");

            return sb.ToString();
        }

        /// <summary>
        /// Processes the specified select builder.
        /// </summary>
        /// <param name="selectBuilder">The select builder.</param>
        /// <returns></returns>
        public override string Process(SelectBuilder selectBuilder)
        {
            // Comprueba que todo est bien			
            StringBuilder sb = new StringBuilder();
            AppendQueryExpression("Select", selectBuilder.Select.ToString(), sb, true);
            AppendQueryExpression("From", selectBuilder.From.ToString(), sb, true);
            AppendQueryExpression("Where", selectBuilder.Where.ToString(), sb, false);
            AppendQueryExpression("GroupBy", selectBuilder.GroupBy.ToString(), sb, false);
            AppendQueryExpression("Having", selectBuilder.Having.ToString(), sb, false);
            AppendQueryExpression("OrderBy", selectBuilder.OrderBy.ToString(), sb, false);
            return sb.ToString();
        }
    }
}