using System;
using System.Text;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Procesador para MsSql2005. Convierte los objetos del QueryBuilder a sentencias SQL compatibles con SQL 2005
    /// </summary>
    internal class MsSql2005Processor : MsSql2000Processor
    {
        /// <summary>
        /// Processes the specified select clause.
        /// </summary>
        /// <param name="selectClause">The select clause.</param>
        /// <returns></returns>
        public override string Process(SelectClause selectClause)
        {
            if (selectClause == null)
                throw new ArgumentNullException("selectClause");

            if (selectClause.EnableCalculateFoundRows)
                throw new InvalidOperationException("Calculate rows not supported by this dialect");

            return ProcessSelectClause(selectClause);
        }

        public override string Process(SelectBuilder selectBuilder)
        {
            if (selectBuilder.Select.Offset <= 0)
            {
                return base.Process(selectBuilder);
            }
            else
            {
                return ProcessSelectBuilderWithOffset(selectBuilder);
            }
        }

        private string ProcessSelectBuilderWithOffset(SelectBuilder selectBuilder)
        {
            int offset = selectBuilder.Select.Offset;

            int firstResult = offset + 1;

            OrderByClause orderByClause = (OrderByClause) selectBuilder.OrderBy.Clone();

            if (orderByClause.Fields.Count == 0)
            {
                orderByClause.Fields.Add("CURRENT_TIMESTAMP");
            }

            string orderBy = orderByClause.ToString();

            string rowCountFieldSql = "ROW_NUMBER() OVER(" + orderBy + ") as RowNumber";
            SqlField rowCountField = new SqlField(rowCountFieldSql);

            string queryName = "[__pagedQuery__]";

            SelectBuilder innerSelectBuilder = (SelectBuilder) selectBuilder.Clone();
            // innerSelectBuilder.Select.Limit = lastResult;

            innerSelectBuilder.Fields.Insert(0, rowCountField);

            innerSelectBuilder.Select.Limit = -1;
            innerSelectBuilder.Select.Offset = 0;
            innerSelectBuilder.OrderBy.Fields.Clear();


            SelectBuilder outerSelectBuilder = new SelectBuilder();
            outerSelectBuilder.TableName = queryName;
            outerSelectBuilder.Fields.Add("*");
            outerSelectBuilder.Where.Filters.Add("RowNumber", SqlOperator.GreaterOrEqual, firstResult);
            if (selectBuilder.Select.Limit >= 0)
            {
                int lastResult = offset + selectBuilder.Select.Limit;
                // Si se suma a Int32.MaxValue un nmero, da la vuelta y comienza con los negativos...
                if (lastResult < 0) lastResult = Int32.MaxValue;

                outerSelectBuilder.Where.Filters.Add("RowNumber", SqlOperator.LessOrEqual, lastResult);
            }

            // outerSelectBuilder.OrderBy = (OrderByClause)orderByClause.Clone();

            string innerSelectBuilderSql = base.Process(innerSelectBuilder);
            string outerSelectBuilderSql = base.Process(outerSelectBuilder);

            StringBuilder commandText = new StringBuilder();
            commandText.AppendFormat("WITH {0} AS (\n", queryName);
            commandText.Append(innerSelectBuilderSql);
            commandText.Append(")\n");
            commandText.Append(outerSelectBuilderSql);

            return commandText.ToString();
        }
    }
}