using System;
using System.Text;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Procesador 
    /// </summary>
    internal class MySqlProcessor : QueryProcessor
    {
        private string _parameterToken = "@";

        /// <summary>
        /// Gets the parameter symbol.
        /// </summary>
        /// <value>The parameter symbol.</value>
        public override string ParameterToken
        {
            get
            {
                return this._parameterToken;
            }
            set
            {
                this._parameterToken = value;
            }
        }

        /// <summary>
        /// Gets the starting quote token.
        /// </summary>
        /// <value>The starting quote token.</value>
        public override string StartingQuoteToken
        {
            get { return "`"; }
        }

        /// <summary>
        /// Gets the ending quote token.
        /// </summary>
        /// <value>The ending quote token.</value>
        public override string EndingQuoteToken
        {
            get { return "`"; }
        }

        /// <summary>
        /// Processes the specified select clause.
        /// </summary>
        /// <param name="selectClause">The select clause.</param>
        /// <returns></returns>
        public override string Process(SelectClause selectClause)
        {
            if (selectClause == null)
                throw new ArgumentNullException("selectClause");

            StringBuilder sb = new StringBuilder();

            sb.Append("SELECT ");

            // Distinct
            if (selectClause.SelectionType == SelectionTypes.Distinct)
            {
                sb.Append("DISTINCT ");
            }

            // CalcFoundRows
            if (selectClause.EnableCalculateFoundRows)
            {
                sb.Append("SQL_CALC_FOUND_ROWS ");
            }

            // Fields
            if (selectClause.Fields.Count > 0)
                sb.Append(selectClause.Fields.ToString());
            else
                sb.Append("*");

            return sb.ToString();
        }

        /// <summary>
        /// Processes the specified select builder.
        /// </summary>
        /// <param name="selectBuilder">The select builder.</param>
        /// <returns></returns>
        public override string Process(SelectBuilder selectBuilder)
        {
            // Comprueba que todo est bien			
            StringBuilder sb = new StringBuilder();
            AppendQueryExpression("Select", selectBuilder.Select.ToString(), sb, true);
            AppendQueryExpression("From", selectBuilder.From.ToString(), sb, true);
            AppendQueryExpression("Where", selectBuilder.Where.ToString(), sb, false);
            AppendQueryExpression("GroupBy", selectBuilder.GroupBy.ToString(), sb, false);
            AppendQueryExpression("Having", selectBuilder.Having.ToString(), sb, false);
            AppendQueryExpression("OrderBy", selectBuilder.OrderBy.ToString(), sb, false);

            if (selectBuilder.Select.Limit > 0)
                AppendQueryExpression("Limit", String.Format("LIMIT {0}", selectBuilder.Select.Limit), sb, false);

            if (selectBuilder.Select.Offset > 0)
                AppendQueryExpression("Offset", String.Format("OFFSET {0}", selectBuilder.Select.Offset), sb, false);

            return sb.ToString();
        }

        protected override string GetDateStringValue(DateTime value)
        {
            return "'" + value.ToString("yyyy-MM-dd HH:mm:ss") + "'";
        }

      
    }
}