using System;
using System.Collections;
using System.Text;

namespace Avanzis.VirtualFraming.Helpers
{
	/// <summary>
	/// Clases para gener consultas SQL
	/// </summary>
	public class QueryBuilder
	{
		private SelectClause select = null;		
		private FromClause from = null;
		private WhereClause where = null;
		private GroupByClause groupBy = null;
		private OrderByClause orderBy = null;

		public SelectClause Select
		{
			get
			{
				if (select == null)
					select = new SelectClause();
				return select;
			}
			set
			{
				select = value;
			}
		}

		public FromClause From
		{
			get
			{
				if (from == null)
					from = new FromClause();
				return from;
			}
			set
			{
				from = value;
			}
		}

		public WhereClause Where
		{
			get
			{
				if (where == null)
					where = new WhereClause();
				return where;
			}
			set
			{
				where = value;
			}
		}

		public GroupByClause GroupBy
		{
			get
			{
				if (groupBy == null)
					groupBy = new GroupByClause();
				return groupBy;
			}
			set
			{
				groupBy = value;
			}
		}

		public OrderByClause OrderBy
		{
			get
			{
				if (orderBy == null)
					orderBy = new OrderByClause();
				return orderBy;
			}
			set
			{
				orderBy = value;
			}
		}


		public QueryBuilder()
		{			
		}	

		public QueryBuilder(string selectFields, string fromExpression)
		{
			this.select = new SelectClause(selectFields);
			this.From = new FromClause(fromExpression);
		}


		public override string ToString()
		{
			// Comprueba que todo est bien			
			StringBuilder sb = new StringBuilder();
			AppendQueryExpression("Select", Select.ToString(), sb, true);
			AppendQueryExpression("From", From.ToString(), sb, true);
			AppendQueryExpression("Where", Where.ToString(), sb, false);
			AppendQueryExpression("GroupBy", GroupBy.ToString(), sb, false);
			AppendQueryExpression("OrderBy", OrderBy.ToString(), sb, false);
			return sb.ToString();
		}

		private void AppendQueryExpression(string expressionName, string expression, StringBuilder sb, bool throwExceptionIfEmpty)
		{
			if (expression != null && expression.Length > 0)
			{
				sb.Append(expression);
				sb.Append("\n");
			}
			else
			{
				if (throwExceptionIfEmpty)
					throw new ArgumentNullException(expressionName, "La clusula '" + expressionName + "' debe estar definida");
			}

		}


		public static string GetJoinTypeString(JoinTypes joinType)
		{
			switch(joinType)
			{
				case JoinTypes.FULL:
					return "FULL JOIN";					
				case JoinTypes.LEFT:
					return "LEFT JOIN";					
				case JoinTypes.RIGHT:
					return "RIGHT JOIN";					
				case JoinTypes.INNER:
				default:
					return "INNER JOIN";					
			}
		}

		public static string GetSqlOperatorString(SqlOperator sqlOperator)
		{
			string operatorString = null;
			switch(sqlOperator)
			{
				case SqlOperator.Contains:
					operatorString = "CONTAINS";
					break;
				case SqlOperator.Distinct:
					operatorString = "!=";
					break;				
				case SqlOperator.Exists:
					operatorString = "EXISTS";
					break;
				case SqlOperator.Greater:
					operatorString = ">";
					break;
				case SqlOperator.GreaterOrEqual:
					operatorString = ">=";
					break;
				case SqlOperator.In:
					operatorString = "IN";
					break;
				case SqlOperator.NotIn:
					operatorString = "NOT IN";
					break;
				case SqlOperator.IsNotNull:
					operatorString = "IS NOT NULL";
					break;
				case SqlOperator.IsNull:
					operatorString = "IS NULL";
					break;
				case SqlOperator.Less:
					operatorString = "<";
					break;
				case SqlOperator.LessOrEqual:
					operatorString = "<=";
					break;
				case SqlOperator.Like:
					operatorString = "LIKE";
					break;
				case SqlOperator.NotLike:
					operatorString = "NOT LIKE";
					break;
				case SqlOperator.Equals:					
				default:
					operatorString = "=";
					break;
			}

			return operatorString;
		}
	}

	
	

	
	

	
	
	

		

		
}
