using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clusula SELECT
    /// </summary>
    public class SelectClause : ICloneable
    {
        private bool enableCalculateFoundRows;
        private int limit = -1;
        private int offset = 0;
        private SelectionTypes selectionType = SelectionTypes.All;
        private SqlFieldCollection selectList;

        /// <summary>
        /// Constructor
        /// </summary>
        public SelectClause()
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="commaSeparatedFields">Campos de la instruccin SELECT separados por ','</param>
        public SelectClause(string commaSeparatedFields)
        {
            if (commaSeparatedFields == null)
                throw new ArgumentNullException("commaSeparatedFields");

            string[] fields = commaSeparatedFields.Trim().Split(',');
            foreach (string field in fields)
            {
                this.Fields.Add(field.Trim());
            }
        }

        /// <summary>
        /// Tipo de seleccin de filas
        /// </summary>
        public SelectionTypes SelectionType
        {
            get { return selectionType; }
            set { selectionType = value; }
        }

        /// <summary>
        /// N mximo de filas que debe devoler
        /// </summary>
        public int Limit
        {
            get { return limit; }
            set { limit = value; }
        }

        /// <summary>
        /// A partir de qu fila debe empezar a devolver resultados
        /// </summary>
        /// <value>The offset.</value>
        public int Offset
        {
            get { return offset; }
            set { offset = value; }
        }

        /// <summary>
        /// Si debe aadir a la clusula SELECT la opcin de calcular el nmero de filas
        /// que se devolveran si no se hubiera establecido un lmite
        /// </summary>
        /// <value>
        /// 	<c>true</c> if [enable caculate found rows]; otherwise, <c>false</c>.
        /// </value>
        /// <remarks>
        /// Slo vlido para MySql
        /// </remarks>
        public bool EnableCalculateFoundRows
        {
            get { return enableCalculateFoundRows; }
            set { enableCalculateFoundRows = value; }
        }

        /// <summary>
        /// Son las columnas que se van a seleccionar para el conjunto de resultados
        /// </summary>
        public SqlFieldCollection Fields
        {
            get
            {
                if (selectList == null)
                    selectList = new SqlFieldCollection();
                return selectList;
            }
        }

        /// <summary>
        /// Representacin SQL de la consulta SELECT
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clasula clon de la actual
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            SelectClause clone = new SelectClause();
            clone.SelectionType = this.SelectionType;
            clone.selectList = this.selectList != null ? (SqlFieldCollection) this.selectList.Clone() : null;
            clone.limit = this.limit;
            clone.offset = this.offset;
            return clone;
        }

        #endregion
    }
}