using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clases para gener consultas SQL
    /// </summary>
    public class UpdateBuilder : SaveBuilder, ICloneable
    {
        private UpdateClause update;
        private WhereClause where;

        /// <summary>
        /// Constructor
        /// </summary>
        public UpdateBuilder()
        {
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla que se actualizar</param>
        public UpdateBuilder(string tableName)
        {
            this.Update = new UpdateClause(tableName);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla qeu se actualizar</param>
        /// <param name="fieldValues">Coleccin de campos/valores</param>
        public UpdateBuilder(string tableName, SqlFieldValueCollection fieldValues)
        {
            this.Update = new UpdateClause(tableName, fieldValues);
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="tableName">Nombre de la tabla</param>
        /// <param name="fieldValues">Coleccin de campos/valores</param>
        /// <param name="singleWhereCondition">Una expresin de filtrado nica</param>
        public UpdateBuilder(string tableName, SqlFieldValueCollection fieldValues, string singleWhereCondition)
        {
            this.Update = new UpdateClause(tableName, fieldValues);
            this.Where.Filters.Add(singleWhereCondition);
        }

        /// <summary>
        /// Clasula UPDATE
        /// </summary>
        public UpdateClause Update
        {
            get
            {
                if (update == null)
                    update = new UpdateClause();
                return update;
            }
            set { update = value; }
        }

        /// <summary>
        /// Clasula WHERE 
        /// </summary>
        public WhereClause Where
        {
            get
            {
                if (where == null)
                {
                    where = new WhereClause();
                    where.Filters.DefaultExpressionType = ExpressionType.Primitive;
                }
                return where;
            }
            set { where = value; }
        }

        /// <summary>
        /// Filtros para realizar la UPDATE
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get { return this.Where.Filters; }
        }

        /// <summary>
        /// Coleccin de Campos/Valores
        /// </summary>
        public override SqlFieldValueCollection FieldValues
        {
            get { return this.Update.FieldValues; }
        }

        /// <summary>
        /// Nombre de la tabla que se va a actualizar
        /// </summary>
        public override string TableName
        {
            get { return this.Update.TableName; }
            set { this.Update.TableName = value; }
        }

        /// <summary>
        /// Devuelve la representacin sql de la consulta UPDATE
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        /// <summary>
        /// Aade un campo con su valor
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="value">The value.</param>
        public void AddFieldValue(string field, object value)
        {
            this.Update.FieldValues.Add(field, value);
        }

        /// <summary>
        /// Aade un campo con su valor
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="value">The value.</param>
        /// <param name="expressionType"> </param>
        public void AddFieldValue(string field, object value, ExpressionType expressionType)
        {
            this.Update.FieldValues.Add(field, value, expressionType);
        }

        /// <summary>
        /// Establece el valor de un campo
        /// </summary>
        /// <param name="field">The field.</param>
        /// <param name="value">The value.</param>
        public void SetFieldValue(string field, object value)
        {
            if (this.Update.FieldValues.ContainsKey(field))
                this.Update.FieldValues[field] = value;
            else
                this.AddFieldValue(field, value);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la consulta actual
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            UpdateBuilder cloned = new UpdateBuilder();

            cloned.update = (UpdateClause) this.Update.Clone();
            cloned.where = (WhereClause) this.Where.Clone();

            return cloned;
        }

        #endregion
    }
}