using System;

namespace Avanzis.QueryBuilder
{
    /// <summary>
    /// Clusula WHERE
    /// </summary>
    public class WhereClause : ICloneable
    {
        private SearchCondition searchCondition;

        /// <summary>
        /// Condicin de bsqueda
        /// </summary>
        public SearchCondition SearchCondition
        {
            get
            {
                if (searchCondition == null)
                {
                    searchCondition = new SearchCondition();
                    searchCondition.Filters.DefaultExpressionType = ExpressionType.Unknown;
                }
                return searchCondition;
            }
            set { searchCondition = value; }
        }

        /// <summary>
        /// Filtros de la condicin de bsqueda
        /// </summary>
        public SearchFiltersCollection Filters
        {
            get { return SearchCondition.Filters; }
        }

        /// <summary>
        /// Union por defecto de la condicin de bsqueda
        /// </summary>
        public FilterUnion Union
        {
            get { return SearchCondition.DefaultFiltersUnion; }
            set { SearchCondition.DefaultFiltersUnion = value; }
        }

        /// <summary>
        /// Si debe negarse la condicin de bsqueda
        /// </summary>
        public bool Negative
        {
            get { return SearchCondition.Negative; }
            set { SearchCondition.Negative = value; }
        }

        /// <summary>
        /// Devuelve la representacin SQL de esta consulta
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return QueryProcessor.Current.Process(this);
        }

        #region Miembros de ICloneable

        /// <summary>
        /// Devuelve un clon de la clasula WHERE
        /// </summary>
        /// <returns></returns>
        public object Clone()
        {
            WhereClause clone = new WhereClause();
            clone.searchCondition = (SearchCondition) this.SearchCondition.Clone();
            return clone;
        }

        #endregion
    }
}