using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;

namespace Avanzis.Helpers.QueryExecutor
{
    /// <summary>
    /// Interfaz que deben implementar aquellos objetos que se encargan de ejecutar una sentencia SQL
    /// </summary>
    public interface IQueryExecutor
    {
        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DataSet ExecuteDataset(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DataSet ExecuteDataset(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes command and returns a Dataset
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DataTable ExecuteDataTable(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes a commands and returns a reader
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DbDataReader ExecuteReader(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters);


        /// <summary>
        /// Executes the reader.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);

        /// <summary>
        /// Executes the reader.
        /// </summary>
        /// <param name="connection">The connection.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        DbDataReader ExecuteReader(DbConnection connection, CommandType commandType, string commandText, bool externalConnection, params DbParameter[] parameters);

        /// <summary>
        /// Executes the non query.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        void ExecuteNonQuery(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters);


        /// <summary>
        /// Executes the command and returns a scalar object
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        object ExecuteScalar(string connectionString, CommandType commandType, string commandText, params DbParameter[] parameters);


        /// <summary>
        /// Executes the command and returns a scalar object
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <param name="commandType">Type of the command.</param>
        /// <param name="commandText">The command text.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        object ExecuteScalar(DbConnection connection, CommandType commandType, string commandText, params DbParameter[] parameters);


        /// <summary>
        /// Gets the db connection.
        /// </summary>
        /// <param name="connectionString">The connection string.</param>
        /// <returns></returns>
        DbConnection GetConnection(string connectionString);

    }
}
