using System;
using System.Configuration;

namespace Avanzis.QueryFramework
{
    /// <summary>
    ///   Configuracin del QueryBuilder
    /// </summary>
    public static class QueryFrameworkConfiguration
    {
        static QueryFrameworkConfiguration()
        {
            Conventions = new Conventions();
        }

        private static Dialects dialect = Dialects.MSSql2005Dialect;

        public static event EventHandler ConfigurationChanged;

        /// <summary>
        ///   Gets or sets the dialect of the Database
        /// </summary>
        /// <value>The dialect.</value>
        public static Dialects Dialect
        {
            get { return dialect; }
            set
            {
                dialect = value;

                if (ConfigurationChanged != null)
                    ConfigurationChanged(null, EventArgs.Empty);
            }
        }

        /// <summary>
        ///   Initializes the framework from web config.
        /// </summary>
        public static void InitializeFromWebConfig()
        {
            if (!String.IsNullOrEmpty(ConfigurationManager.AppSettings["QueryFrameworkDialect"]))
                Dialect = (Dialects) Enum.Parse(typeof (Dialects), ConfigurationManager.AppSettings["QueryFrameworkDialect"]);
        }

        /// <summary>
        ///   Returns true if current Database Dialecs allows offset in Select queries
        /// </summary>
        /// <returns></returns>
        public static Boolean DialectAllowsOffset
        {
            get
            {
                switch (Dialect)
                {
                    case Dialects.MSSql2005Dialect:
                    case Dialects.MySql:
                        return true;
                    case Dialects.MSSql2000Dialect:
                        return false;
                    case Dialects.OleDb:
                        return false;
                    default:
                        return false;
                }
            }
        }

        public static Conventions Conventions { get; set; }
    }
}