using System;

namespace Avanzis.QueryObjects
{
    /// <summary>
    /// Propiedad que se persistir en la bbdd
    /// </summary>
    [AttributeUsage(AttributeTargets.Property, AllowMultiple = false), Serializable]
    public class PropertyAttribute : ColumnPropertyAttribute
    {
        private bool unique;
        private bool notNull;
        private bool update = true;
        private bool insert = true;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:PropertyAttribute"/> class.
        /// </summary>
        public PropertyAttribute()
            : base()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:PropertyAttribute"/> class.
        /// </summary>
        /// <param name="columnName">Name of the column.</param>
        public PropertyAttribute(string columnName) 
            : base(columnName)
        {
        }

        public PropertyAttribute(string columnName, string columnType)
            : base(columnName, columnType)
        {
        }

        /// <summary>
        /// Gets or sets a value indicating whether this <see cref="T:PropertyAttribute"/> is unique.
        /// </summary>
        /// <value><c>true</c> if unique; otherwise, <c>false</c>.</value>
        public bool Unique
        {
            get { return unique; }
            set { unique = value; }
        }

        /// <summary>
        /// Gets or sets a value indicating whether [not null].
        /// </summary>
        /// <value><c>true</c> if [not null]; otherwise, <c>false</c>.</value>
        public bool NotNull
        {
            get { return notNull; }
            set { notNull = value; }
        }

        public bool Update
        {
            get { return update; }
            set { update = value; }
        }

        public bool Insert
        {
            get { return insert; }
            set { insert = value; }
        }
       
    }
}
